@@ -79,10 +79,13 @@ public class BTPeerIDByteDecoder {
       if( (decoded = decodeAzStyle( peerID, "TR", "Transmission" )) != null ) return decoded;
       if( (decoded = decodeAzStyle( peerID, "CD", "CTorrent" )) != null ) return decoded;      
       if( (decoded = decodeAzStyle( peerID, "RT", "Retriever" )) != null ) return decoded;      
+      if( (decoded = decodeAzStyle( peerID, "LP", "Lphant" )) != null ) return decoded;
+      if( (decoded = decodeAzStyle( peerID, "PC", "CacheLogic" )) != null ) return decoded; 
       
       if( (decoded = decodeTornadoStyle( peerID, "T", "BitTornado" )) != null ) return decoded;
       if( (decoded = decodeTornadoStyle( peerID, "A", "ABC" )) != null ) return decoded;
       if( (decoded = decodeTornadoStyle( peerID, "O", "Osprey permaseed" )) != null ) return decoded;
+      if( (decoded = decodeTornadoStyle( peerID, "R", "Tribler" )) != null ) return decoded;
      
       if( (decoded = decodeMainlineStyle( peerID, "M", "Mainline" )) != null ) return decoded;
       
@@ -109,6 +112,8 @@ public class BTPeerIDByteDecoder {
       if( (decoded = decodeSimpleStyle( peerID, 0, "a00---0", "Swarmy" )) != null ) return decoded;
       if( (decoded = decodeSimpleStyle( peerID, 0, "a02---0", "Swarmy" )) != null ) return decoded;
       if( (decoded = decodeSimpleStyle( peerID, 0, "T00---0", "Teeweety" )) != null ) return decoded;
+      
+      if( (decoded = decodeSimpleStyle( peerID, 0, "10-------", "JVtorrent" )) != null ) return decoded;
 
       String burst = new String(peerID, 0, 5, Constants.BYTE_ENCODING);
       if( burst.equals( "Mbrst" ) ) {
@@ -445,6 +450,9 @@ public class BTPeerIDByteDecoder {
   
   private static String decodeMainlineStyle( byte[] id, String ident, String name ) {
     try {
+    	
+    	// Mx-y-z--
+    	// where version = x.y.z
       if ( (id[2] == (byte)45) && (id[4] == (byte)45) && (id[6] == (byte)45) && (id[7] == (byte)45) ) {
         String decoded = new String( id, 0, 1, Constants.BYTE_ENCODING );
         if( decoded.equals( ident ) ) {
@@ -454,6 +462,17 @@ public class BTPeerIDByteDecoder {
           return name + " " + v1 + "." + v2 + "." + v3;
         }
       }
+    	// Mx-yy-z-
+    	// where version = x.yy.z
+      if ( (id[2] == (byte)45) && (id[5] == (byte)45) && (id[7] == (byte)45) ) {
+        String decoded = new String( id, 0, 1, Constants.BYTE_ENCODING );
+        if( decoded.equals( ident ) ) {
+          String v1 = new String( id, 1, 1, Constants.BYTE_ENCODING );
+          String v2 = new String( id, 3, 2, Constants.BYTE_ENCODING );
+          String v3 = new String( id, 6, 1, Constants.BYTE_ENCODING );
+          return name + " " + v1 + "." + v2 + "." + v3;
+        }
+      }
     }
     catch( Exception e ) {  return null;  }
     return null;
