@@ -33,6 +33,7 @@ import java.io.*;
 
 import org.gudy.azureus2.core3.util.AEMonitor;
 import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.ThreadPool;
 
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
@@ -123,9 +124,11 @@ UPnPImpl
 	rootDiscovered(
 		final NetworkInterface		network_interface,
 		final InetAddress			local_address,
+		final String				usn,
 		final URL					location )
 
 	{
+		
 			// we need to take this operation off the main thread as it can take some time. This is a single
 			// concurrency queued thread pool so things get done serially in the right order
 		
@@ -140,7 +143,7 @@ UPnPImpl
 					try{
 						rd_listeners_mon.enter();
 
-						old_root_device = (UPnPRootDeviceImpl)root_locations.get( location.getHost());
+						old_root_device = (UPnPRootDeviceImpl)root_locations.get( usn );
 						
 					}finally{
 						
@@ -149,42 +152,19 @@ UPnPImpl
 					
 					if ( old_root_device != null ){
 				
-							// device is still there with same IP, however 
+							// we remember one route to the device - if the network interfaces change
+							// we do a full reset so we don't need to deal with that here
 						
+						if ( !old_root_device.getNetworkInterface().getName().equals( network_interface.getName())){
+							
+							return;
+						}
+						
+							// check that the device's location is the same
+												
 						if ( old_root_device.getLocation().equals( location )){		
-							
-								// 1) port of UPnP device might have changed (it does on mine when enabling/disabling UPnP)
-								// 2) our local IP might have changed (DHCP reassignment)
 
-							if ( old_root_device.getLocalAddress().equals( local_address )){
-							
-								// everythings the same, nothing to do
-							
-								return;
-							}
-						
-								// an alternative situation is where the same device is discovered by two network interfaces
-								// (see https://sourceforge.net/forum/message.php?msg_id=2912370 )
-								// if this is the case we just use the first NI through which it was discovered and
-								// map to that NI's local address
-						
-								// use the interfate.getName (e.g. eth0) as the "equals" method includes the addresses and we want to
-								// detect when an address has changed
-							
-							if ( !old_root_device.getNetworkInterface().getName().equals( network_interface.getName())){
-							
-								String	msg =  "UPnP: secondary route to = " + location + ", local = " + local_address.toString() + " - using initial network interface (" + 
-													old_root_device.getNetworkInterface();
-
-								if ( !secondary_route_log.equals( msg )){
-								
-									secondary_route_log	= msg;
-									
-									log( msg );
-								}
-				
-								return;
-							}
+							return;
 						}
 					}
 					
@@ -198,7 +178,7 @@ UPnPImpl
 							
 							rd_listeners_mon.enter();
 
-							root_locations.remove( location.getHost());
+							root_locations.remove( usn );
 						
 						}finally{
 							
@@ -208,20 +188,44 @@ UPnPImpl
 						old_root_device.destroy( true );
 					}
 		
-					log( "UPnP: root discovered = " + location + ", local = " + local_address.toString() );
+					log( "UPnP: root discovered: usn=" + usn + ", location=" + location + ", ni=" + network_interface.getName() + ",local=" + local_address.toString() );
+					
+					List	listeners;
 					
 					try{
-						UPnPRootDeviceImpl new_root_device = new UPnPRootDeviceImpl( UPnPImpl.this, network_interface, local_address, location );
-					
-						final List	listeners;
+						rd_listeners_mon.enter();
 						
+						listeners = new ArrayList( rd_listeners );
+						
+					}finally{
+						
+						rd_listeners_mon.exit();
+					}
+					
+					for (int i=0;i<listeners.size();i++){
+						
+						try{
+							if ( !((UPnPListener)listeners.get(i)).deviceDiscovered( usn, location )){
+								
+								return;
+							}
+							
+						}catch( Throwable e ){
+							
+							Debug.printStackTrace(e);
+						}
+					}				
+
+					try{
+						UPnPRootDeviceImpl new_root_device = new UPnPRootDeviceImpl( UPnPImpl.this, network_interface, local_address, usn, location );
+											
 						try{
 							rd_listeners_mon.enter();
 							
-							root_locations.put( location.getHost(), new_root_device );
-			
+							root_locations.put( usn, new_root_device );
+								
 							listeners = new ArrayList( rd_listeners );
-							
+
 						}finally{
 							
 							rd_listeners_mon.exit();
@@ -229,7 +233,13 @@ UPnPImpl
 			
 						for (int i=0;i<listeners.size();i++){
 							
-							((UPnPListener)listeners.get(i)).rootDeviceFound( new_root_device );	
+							try{
+								((UPnPListener)listeners.get(i)).rootDeviceFound( new_root_device );
+								
+							}catch( Throwable e ){
+								
+								Debug.printStackTrace(e);
+							}
 						}
 					
 					}catch( UPnPException e ){
@@ -244,9 +254,10 @@ UPnPImpl
 	
 	public void
 	rootAlive(
+		String		usn,
 		URL			location )
 	{
-		UPnPRootDeviceImpl root_device = (UPnPRootDeviceImpl)root_locations.get( location.getHost());
+		UPnPRootDeviceImpl root_device = (UPnPRootDeviceImpl)root_locations.get( usn );
 			
 		if ( root_device == null ){
 			
@@ -257,7 +268,7 @@ UPnPImpl
 	public void
 	rootLost(
 		final InetAddress	local_address,
-		final URL			location )
+		final String		usn )
 	{
 			// we need to take this operation off the main thread as it can take some time
 		
@@ -267,13 +278,13 @@ UPnPImpl
 				public void
 				runSupport()
 				{
-					UPnPRootDeviceImpl	root_device;
-				
+					UPnPRootDeviceImpl	root_device	= null;
+									
 					try{
 						rd_listeners_mon.enter();
 			
-						root_device = (UPnPRootDeviceImpl)root_locations.remove( location.getHost());
-						
+						root_device = (UPnPRootDeviceImpl)root_locations.remove( usn );
+				
 					}finally{
 						
 						rd_listeners_mon.exit();
@@ -283,14 +294,21 @@ UPnPImpl
 						
 						return;
 					}
-					
-					log( "UPnP: root lost = " + location +", local = " + local_address.toString() );
+									
+					log( "UPnP: root lost: usn=" + usn + ", location=" + root_device.getLocation() + ", ni=" + root_device.getNetworkInterface().getName() + ",local=" + root_device.getLocalAddress().toString());
 				
 					root_device.destroy( false );
 				}
 			});
 	}
 	
+	public void
+	interfaceChanged(
+		NetworkInterface	network_interface )
+	{
+		reset();
+	}
+	
 	public void
 	reset()
 	{
@@ -361,6 +379,8 @@ UPnPImpl
 			
 			LineNumberReader	lnr = new LineNumberReader( new InputStreamReader( is, "UTF-8" ));
 			
+			Set	ignore_map = null;
+			
 			while( true ){
 				
 				String	line = lnr.readLine();
@@ -370,7 +390,37 @@ UPnPImpl
 					break;
 				}
 				
-				data.append( line.trim() + "\n" );	
+					// remove any obviously invalid characters - I've seen some routers generate stuff like
+					// 0x18 which stuffs the xml parser with "invalid unicode character"
+				
+				for (int i=0;i<line.length();i++){
+					
+					char	c = line.charAt(i);
+				
+					if ( c < 0x20 && c != '\r' && c != '\t' ){
+						
+						data.append( ' ' );
+						
+						if ( ignore_map == null ){
+							
+							ignore_map = new HashSet();
+						}
+						
+						Character	cha = new Character(c);
+						
+						if ( !ignore_map.contains( cha )){
+						
+							ignore_map.add( cha );
+							
+							adapter.trace( "    ignoring character(s) " + (int)c + " in xml response" );
+						}
+					}else{
+						
+						data.append( c );
+					}
+				}
+				
+				data.append( "\n" );				
 			}
 				
 			String	data_str = data.toString();
@@ -442,20 +492,27 @@ UPnPImpl
 	{
 		SimpleXMLParserDocument	res;
 		
-		try{
-			res =  performSOAPRequest( service, soap_action, request, true );
-				
-			http_calls_ok++;
-			
-		}catch( IOException e ){
+		if ( service.getDirectInvocations()){
 			
 			res = performSOAPRequest( service, soap_action, request, false );
+
+		}else{
 			
-			direct_calls_ok++;
-			
-			if ( direct_calls_ok == 1 ){
+			try{
+				res =  performSOAPRequest( service, soap_action, request, true );
+					
+				http_calls_ok++;
 				
-				log( "Invocation via http connection failed (" + e.getMessage() + ") but socket connection succeeded" );
+			}catch( IOException e ){
+				
+				res = performSOAPRequest( service, soap_action, request, false );
+				
+				direct_calls_ok++;
+				
+				if ( direct_calls_ok == 1 ){
+					
+					log( "Invocation via http connection failed (" + e.getMessage() + ") but socket connection succeeded" );
+				}
 			}
 		}
 		
@@ -759,7 +816,19 @@ UPnPImpl
 		
 		for (int i=0;i<old_locations.size();i++){
 			
-			l.rootDeviceFound(((UPnPRootDevice)old_locations.get(i)));
+			UPnPRootDevice	device = (UPnPRootDevice)old_locations.get(i);
+			
+			try{
+				
+				if ( l.deviceDiscovered( device.getUSN(), device.getLocation())){
+					
+					l.rootDeviceFound(device);
+				}
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
 		}
 	}
 		
@@ -767,7 +836,15 @@ UPnPImpl
 	removeRootDeviceListener(
 		UPnPListener	l )
 	{
-		rd_listeners.remove( l );
+		try{
+			this_mon.enter();
+
+			rd_listeners.remove( l );
+			
+		}finally{
+			
+			this_mon.exit();
+		}
 	}
 	
 	public static void
@@ -780,6 +857,14 @@ UPnPImpl
 			upnp.addRootDeviceListener(
 					new UPnPListener()
 					{
+						public boolean
+						deviceDiscovered(
+							String		USN,
+							URL			location )
+						{
+							return( true );
+						}
+						
 						public void
 						rootDeviceFound(
 							UPnPRootDevice		device )
