@@ -48,13 +48,13 @@ import org.gudy.azureus2.core3.peer.*;
 
 import org.gudy.azureus2.core3.tracker.protocol.*;
 import org.gudy.azureus2.core3.tracker.protocol.udp.*;
-import org.gudy.azureus2.core3.tracker.util.impl.*;
+import org.gudy.azureus2.core3.tracker.util.TRTrackerUtils;
 
 import org.gudy.azureus2.plugins.clientid.*;
 import org.gudy.azureus2.plugins.download.*;
 import org.gudy.azureus2.pluginsimpl.local.clientid.ClientIDManagerImpl;
 
-import com.aelitis.azureus.core.networkmanager.NetworkManager;
+import com.aelitis.azureus.core.networkmanager.impl.udp.UDPNetworkManager;
 import com.aelitis.net.udp.uc.PRUDPPacket;
 import com.aelitis.net.udp.uc.PRUDPPacketHandler;
 import com.aelitis.net.udp.uc.PRUDPPacketHandlerException;
@@ -72,7 +72,6 @@ import com.aelitis.net.udp.uc.PRUDPPacketRequest;
 public class 
 TRTrackerBTAnnouncerImpl
 	extends TRTrackerAnnouncerImpl
-	implements ParameterListener
 {
 	
 		
@@ -90,16 +89,17 @@ TRTrackerBTAnnouncerImpl
 	private static Map			tracker_report_map	= new HashMap();
 	
     
-	protected TOTorrent				torrent;
+	private TOTorrent				torrent;
 	
-	protected TimerEvent				current_timer_event;
+	private TimerEvent				current_timer_event;
 	private TimerEventPerformer		timer_event_action;
 	
-	protected int					tracker_state 			= TS_INITIALISED;
+	protected int				tracker_state 			= TS_INITIALISED;
 	private String				tracker_status_str		= "";
 	private TRTrackerAnnouncerResponse	last_response			= null;
 	private long				last_update_time_secs;
-	protected long				current_time_to_wait_secs;
+	private long				current_time_to_wait_secs;
+	private boolean				manual_control;
   
 	private long min_interval = 0;
   
@@ -107,6 +107,7 @@ TRTrackerBTAnnouncerImpl
     private long failure_time_last_updated = 0;
 	
 	private boolean			stopped;
+	private boolean			stopped_for_queue;
 	private boolean			completed;
 	private boolean			complete_reported	= false;
 	
@@ -121,7 +122,7 @@ TRTrackerBTAnnouncerImpl
      
   	private URL lastUsedUrl;
     
-  	private byte[]				torrent_hash;
+  	private HashWrapper			torrent_hash;
   	
 	private String	last_tracker_message;		// per torrent memory
 	
@@ -139,28 +140,16 @@ TRTrackerBTAnnouncerImpl
 	private String tracker_id = "";
   
   
-  
-	private int			port_number;
-	private String 		port;
+ 
 	private String 		ip_override;
-	private int			port_override	= -1;
 	private String[]	peer_networks;
 		
 	private TRTrackerAnnouncerDataProvider 	announce_data_provider;
 	
 	protected AEMonitor this_mon 	= new AEMonitor( "TRTrackerBTAnnouncer" );
 
-	private static final boolean	socks_peer_inform;
-
 	protected boolean	destroyed;
-	
-	
-	static{
-	 	socks_peer_inform	= 	
-	  		COConfigurationManager.getBooleanParameter("Proxy.Data.Enable", false)&&
-	 		COConfigurationManager.getBooleanParameter("Proxy.Data.SOCKS.inform", true );
-	 }
-	
+		
 
 
 	static final String chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
@@ -181,12 +170,14 @@ TRTrackerBTAnnouncerImpl
   public 
   TRTrackerBTAnnouncerImpl(
    	TOTorrent		_torrent,
-	String[]		_peer_networks ) 
+	String[]		_peer_networks,
+	boolean			_manual ) 
   	
   	throws TRTrackerAnnouncerException
   {
   	torrent			= _torrent;
   	peer_networks	= _peer_networks;
+  	manual_control	= _manual;
   	
 		//Get the Tracker url
 		
@@ -216,9 +207,9 @@ TRTrackerBTAnnouncerImpl
 	
 	try {
 	
-		torrent_hash = _torrent.getHash();
+		torrent_hash = _torrent.getHashWrapper();
 				
-		this.info_hash += URLEncoder.encode(new String(torrent_hash, Constants.BYTE_ENCODING), Constants.BYTE_ENCODING).replaceAll("\\+", "%20");
+		this.info_hash += URLEncoder.encode(new String(torrent_hash.getBytes(), Constants.BYTE_ENCODING), Constants.BYTE_ENCODING).replaceAll("\\+", "%20");
 	  
 		this.tracker_peer_id_str += URLEncoder.encode(new String(tracker_peer_id, Constants.BYTE_ENCODING), Constants.BYTE_ENCODING).replaceAll("\\+", "%20");
 	  
@@ -234,11 +225,6 @@ TRTrackerBTAnnouncerImpl
 		
 		throw( new TRTrackerAnnouncerException( "TRTrackerAnnouncer: URL encode fails"));	
 	}
-
-  
-	COConfigurationManager.addParameterListener("TCP.Announce.Port",this);
-	
-	setPort();
 	   
 	timer_event_action =  
 		new TimerEventPerformer()
@@ -247,6 +233,13 @@ TRTrackerBTAnnouncerImpl
 			perform(
 				TimerEvent	this_event )
 			{
+				if ( manual_control ){
+					
+					requestUpdateSupport();
+					
+					return;
+				}
+				
 				long	secs_to_wait = getErrorRetryInterval();
 							
 				try{
@@ -325,63 +318,6 @@ TRTrackerBTAnnouncerImpl
 			Logger.log(new LogEvent(torrent, LOGID,
 					"Tracker Announcer Created using url : " + trackerURLListToString()));
   }
-	
-	public void
-	portChanged(
-		int		_new_port )
-	{
-		setPort();
-		
-		update( true );
-	}
-	
-  	protected void
-	setPort()
-  	{
-   			// we currently don't support incoming connections when SOCKs proxying
-  		
-  		int	port_num;
-  		
-  		if ( socks_peer_inform ){
-  			
-  			port_num	= 0;
-  			
-  		}else{
-  		
-  			port_num	= NetworkManager.getSingleton().getTCPListeningPortNumber();
-  		}
-  		
- 		if ( port_override != -1 ){
- 			
- 			port_num = port_override;
- 			
- 		}else{
-
-	  		String portOverride = COConfigurationManager.getStringParameter("TCP.Announce.Port","");
-	  		
-	  		if(! portOverride.equals("")) {
-	  		  
-	  			try{
-	  				port_num = Integer.parseInt( portOverride );
-	  				
-	  			}catch( Throwable e ){
-	  				
-	  				Debug.printStackTrace(e);
-	  			}
-	  		}
- 		}
-  		  
- 		port_number	= port_num;
- 		
-  		port = "&port=" + port_num;
-  		  
-  		  	//  BitComet extension for no incoming connections
-  		
-  		if ( port_num == 0 ){
-  			
-  			port += "&hide=1";
-  		}		
-  	}
   	
 	protected long
 	getAdjustedSecsToWait()
@@ -400,9 +336,19 @@ TRTrackerBTAnnouncerImpl
 	  		if( failure_added_time < 900 )  failure_added_time = 900;
   			secs_to_wait = getErrorRetryInterval();
   			
+    		if (Logger.isEnabled()) {
+					Logger.log(new LogEvent(torrent, LOGID,
+							"MIN INTERVAL CALC: tracker reported error, " + 
+							"adjusting to error retry interval"));
+    		}
 	  	}
 	  	else {	//tracker is OFFLINE
 	  		secs_to_wait = getErrorRetryInterval();
+    		if (Logger.isEnabled()) {
+					Logger.log(new LogEvent(torrent, LOGID,
+							"MIN INTERVAL CALC: tracker seems to be offline, " + 
+							"adjusting to error retry interval"));
+    		}
 	  	}
 							
 	  }
@@ -414,8 +360,14 @@ TRTrackerBTAnnouncerImpl
 							"MIN INTERVAL CALC: override, perc = 0"));
       	return REFRESH_MINIMUM_SECS;
       }
-							
-      secs_to_wait = (secs_to_wait * rd_override_percentage) /100;
+
+      if (rd_override_percentage != 100) {
+      	secs_to_wait = (secs_to_wait * rd_override_percentage) /100;
+    		if (Logger.isEnabled()) {
+					Logger.log(new LogEvent(torrent, LOGID,
+							"MIN INTERVAL CALC: override, perc = " + rd_override_percentage));
+    		}
+      }
 									
       if ( secs_to_wait < REFRESH_MINIMUM_SECS ){
 	  			
@@ -557,7 +509,7 @@ TRTrackerBTAnnouncerImpl
 
         long	effective_min = min_interval_override>0?min_interval_override:REFRESH_MINIMUM_SECS;
         
-		if( force || ( now - last_update_time_secs >= effective_min )){
+		if ( manual_control || force || ( now - last_update_time_secs >= effective_min )){
 			
 		  requestUpdate();
 		}
@@ -576,10 +528,12 @@ TRTrackerBTAnnouncerImpl
 	}
 	
 	public void
-	stop()
+	stop(
+		boolean	for_queue )
 	{
-		stopped	= true;
-        		
+		stopped				= true;
+        stopped_for_queue	= for_queue;
+        
 		requestUpdate();
 	}
 	
@@ -717,23 +671,10 @@ TRTrackerBTAnnouncerImpl
       
 					tracker_status_str = MessageText.getString("PeerManager.status.offline"); 
       		      
-					String	reason = response.getFailureReason();
-      		
-					if ( reason != null ){
-      			
-						tracker_status_str += " (" + reason + ")";		
-					}
 				}else if ( rs == TRTrackerAnnouncerResponse.ST_REPORTED_ERROR ){
 
 					tracker_status_str = MessageText.getString("PeerManager.status.error"); 
 	      		      
-					String	reason = response.getFailureReason();
-      		
-					if ( reason != null ){
-      			
-						tracker_status_str += " (" + reason + ")";		
-					}
-			
 						// move state back to initialised to next time around a "started"
 						// event it resent. Required for trackers like 123torrents.com that
 						// will fail peers that don't start with a "started" event after a 
@@ -745,6 +686,12 @@ TRTrackerBTAnnouncerImpl
 	    	       	        	
 					tracker_status_str = MessageText.getString("PeerManager.status.ok"); //set the status      //$NON-NLS-1$
 				}
+
+				String	reason = response.getAdditionalInfo();
+    		
+				if ( reason != null ){
+					tracker_status_str += " (" + reason + ")";		
+				}
 				
 				last_response = response;
 				
@@ -882,29 +829,53 @@ TRTrackerBTAnnouncerImpl
 		
 		for (int j = 0 ; j < urls.size() ; j++) {
 			
-		  URL url = (URL)urls.get(j);
+		  URL original_url = (URL)urls.get(j);
 		  
-		  lastUsedUrl = url;
+		  lastUsedUrl = original_url;
 		   
 		  URL	request_url = null;
 		  
 		  try{
 		  
-		  	request_url = constructUrl(evt,url);
-			  					  	  			  
-			TRTrackerAnnouncerResponseImpl resp = decodeTrackerResponse( url, updateOld(request_url));
-			  
+		  	request_url = constructUrl(evt,original_url);
+			  			
+		  	URL[]	tracker_url = { original_url };
+		  	
+		  	byte[]	result_bytes = updateOld( tracker_url, request_url);
+		  	
+		  	lastUsedUrl = tracker_url[0];	// url may have redirected, use this value as it will be correct
+		  	
+			TRTrackerAnnouncerResponseImpl resp = decodeTrackerResponse( lastUsedUrl, result_bytes );
+			  	    	
 		    if ( resp.getStatus() == TRTrackerAnnouncerResponse.ST_ONLINE ){
 					
+		    	try{
+		    			// tracker looks ok, make any redirection permanent
+		    		
+		    		if ( !original_url.toString().equals(lastUsedUrl.toString())){
+		    	
+		    			if (Logger.isEnabled())
+							Logger.log(new LogEvent(torrent, LOGID,
+									"announce url permanently redirected: old = " + original_url + ", new = " + lastUsedUrl ));
+						
+					
+						TorrentUtils.replaceAnnounceURL( torrent, original_url, lastUsedUrl );
+						
+					}
+		    	}catch( Throwable e ){
+		    		
+		    		Debug.printStackTrace(e);
+		    	}
+
 	            urls.remove(j);
 	            	
-	            urls.add(0,url);
+	            urls.add(0, lastUsedUrl );	
 	            	
 	            trackerUrlLists.remove(i);
 	            	
 	            trackerUrlLists.add(0,urls);            
 	            
-	            informURLChange( url, false );
+	            informURLChange( lastUsedUrl, false );
 	            	
 	            	//and return the result
 	            		
@@ -920,7 +891,7 @@ TRTrackerBTAnnouncerImpl
 		  	
 		  	last_failure_resp = 
 		  		new TRTrackerAnnouncerResponseImpl( 
-		  				url,
+		  				original_url,
 		  				torrent_hash,
 		  				TRTrackerAnnouncerResponse.ST_OFFLINE, 
 						getErrorRetryInterval(), 
@@ -930,7 +901,7 @@ TRTrackerBTAnnouncerImpl
 		  			  	
 		  	last_failure_resp = 
 		  		new TRTrackerAnnouncerResponseImpl(
-		  				url,
+		  				original_url,
 		  				torrent_hash,
 		  				TRTrackerAnnouncerResponse.ST_OFFLINE, 
 						getErrorRetryInterval(), 
@@ -978,6 +949,7 @@ TRTrackerBTAnnouncerImpl
 
  	private byte[] 
  	updateOld(
+ 		URL[]		tracker_url,
  		URL 		reqUrl )
   
   		throws Exception
@@ -1008,7 +980,7 @@ TRTrackerBTAnnouncerImpl
 		  			
 		  		}else{
 		  			
-		  			failure_reason = announceHTTP( reqUrl, message );
+		  			failure_reason = announceHTTP( tracker_url, reqUrl, message );
 		  			
 		  		}
 					// if we've got some kind of response then return it
@@ -1077,14 +1049,15 @@ TRTrackerBTAnnouncerImpl
  	
  	protected String
  	announceHTTP(
+ 		URL[]					tracker_url,	// overwritten if redirected
  		URL						reqUrl,
  		ByteArrayOutputStream	message )
  	
  		throws IOException
- 	{
- 		TRTrackerUtilsImpl.checkForBlacklistedURLs( reqUrl );
+ 	{ 		
+ 		TRTrackerUtils.checkForBlacklistedURLs( reqUrl );
  		
- 		reqUrl = TRTrackerUtilsImpl.adjustURLForHosting( reqUrl );
+ 		reqUrl = TRTrackerUtils.adjustURLForHosting( reqUrl );
  		
  		reqUrl = AddressUtils.adjustURL( reqUrl );
  		
@@ -1158,6 +1131,34 @@ TRTrackerBTAnnouncerImpl
  				
  				is = con.getInputStream();
  				
+ 				String	resulting_url_str = con.getURL().toString();
+ 				
+ 				if ( !reqUrl.toString().equals( resulting_url_str )){
+ 					
+ 						// some kind of redirect has occurred. Unfortunately we can't get at the underlying
+ 						// redirection reason (temp, perm etc) so we support the use of an explicit indicator
+ 						// in the resulting url
+ 					
+ 					String	marker = "permredirect=1";
+ 					
+ 					int	pos = resulting_url_str.indexOf( marker );
+ 				
+ 					if ( pos != -1 ){
+ 						
+ 						pos = pos-1;	// include the '&' or '?'
+ 						
+ 						try{
+ 							URL	redirect_url = 
+ 								new URL( resulting_url_str.substring(0,pos));
+ 							
+  							tracker_url[0]	= redirect_url;
+						
+ 						}catch( Throwable e ){
+ 							Debug.printStackTrace(e);
+ 						}
+ 					}
+ 				}
+ 				
  				String encoding = con.getHeaderField( "content-encoding");
  				
  				boolean	gzip = encoding != null && encoding.equalsIgnoreCase("gzip");
@@ -1264,7 +1265,7 @@ TRTrackerBTAnnouncerImpl
  	
  		throws IOException
  	{
- 		reqUrl = TRTrackerUtilsImpl.adjustURLForHosting( reqUrl );
+ 		reqUrl = TRTrackerUtils.adjustURLForHosting( reqUrl );
 
  		String	failure_reason = null;
 		
@@ -1276,7 +1277,7 @@ TRTrackerBTAnnouncerImpl
  				 auth = SESecurityManager.getPasswordAuthentication( UDP_REALM, reqUrl );
  			}
  						
- 			PRUDPPacketHandler handler = PRUDPPacketHandlerFactory.getHandler( NetworkManager.getSingleton().getTCPListeningPortNumber() );
+ 			PRUDPPacketHandler handler = PRUDPPacketHandlerFactory.getHandler( UDPNetworkManager.getSingleton().getUDPNonDataListeningPortNumber());
  			
  			InetSocketAddress destination = new InetSocketAddress(reqUrl.getHost(),reqUrl.getPort()==-1?80:reqUrl.getPort());
  			
@@ -1340,7 +1341,7 @@ TRTrackerBTAnnouncerImpl
 			 				}
 			 				
 			 				announce_request.setDetails(
-			 					torrent_hash,
+			 					torrent_hash.getBytes(),
 			 					tracker_peer_id,
 								getLongURLParam( url_str, "downloaded" ), 
 								event,
@@ -1393,7 +1394,7 @@ TRTrackerBTAnnouncerImpl
 			 				}
 			 				
 			 				announce_request.setDetails(
-			 					torrent_hash,
+			 					torrent_hash.getBytes(),
 			 					tracker_peer_id,
 								getLongURLParam( url_str, "downloaded" ), 
 								event,
@@ -1591,7 +1592,10 @@ TRTrackerBTAnnouncerImpl
   	
   	request.append(info_hash);
   	request.append(tracker_peer_id_str);
-  	request.append(port);
+  	
+	String	port_details = TRTrackerUtils.getPortsForURL();
+
+  	request.append(port_details);
   	request.append("&uploaded=").append(announce_data_provider.getTotalSent());
   	request.append("&downloaded=").append(announce_data_provider.getTotalReceived());
   	request.append("&left=").append(announce_data_provider.getRemaining());
@@ -1611,6 +1615,10 @@ TRTrackerBTAnnouncerImpl
     	
     	request.append("&numwant=0");
     	
+    	if ( stopped_for_queue ){
+    		
+    		request.append( "&azq=1" );
+    	}
     }else {
     	
       //calculate how many peers we should ask for
@@ -1862,29 +1870,6 @@ TRTrackerBTAnnouncerImpl
 	{
 		ip_override = null;
 	}
-		
-	public void
-	setPortOverride(
-		int		port )
-	{
-		port_override	= port;
-		
-		setPort();
-	}
-	
-	public void
-	clearPortOverride()
-	{
-		port_override	= -1;
-		
-		setPort();
-	}
-	
-	public int
-	getPort()
-	{
-		return( port_number );
-	}
 	
 	private void 
 	constructTrackerUrlLists(
@@ -2115,6 +2100,7 @@ TRTrackerBTAnnouncerImpl
 			             tracker_id = new String( trackerid );
 			           }
 			           
+				    byte[]	crypto_flags = (byte[])metaData.get( "crypto_flags" );
            						
 						//build the list of peers
 					List valid_meta_peers = new ArrayList();
@@ -2128,10 +2114,19 @@ TRTrackerBTAnnouncerImpl
 				    	
 						List meta_peers = (List)meta_peers_peek;
 						 					 
-						
 							//for every peer
+						
 						int peers_length = meta_peers.size();
 							
+						if ( crypto_flags != null && peers_length != crypto_flags.length ){
+							
+							crypto_flags = null;
+							
+                           	if (Logger.isEnabled())
+                        		Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
+                        				"Invalid crypto_flags returned: length mismatch" ));
+						}
+						
 						for (int i = 0; i < peers_length; i++) {
 							 	
 							Map peer = (Map) meta_peers.get(i);
@@ -2139,7 +2134,7 @@ TRTrackerBTAnnouncerImpl
 							Object s_peerid	= peer.get("peer id"); 
 							Object s_ip		= peer.get("ip"); 
 							Object s_port	= peer.get("port"); 
-													
+												
 								// Assert that all ip and port are available
 							
 							if ( s_ip != null && s_port != null ){
@@ -2158,12 +2153,12 @@ TRTrackerBTAnnouncerImpl
                                 if (peer_port <0)
                                     peer_port +=65536;
 				                
-                if (peer_port < 0 || peer_port > 65535) {
-                	if (Logger.isEnabled())
-										Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
-												"Invalid peer port given: " + ip + ": " + peer_port));
-                  continue;
-                }
+                                if (peer_port < 0 || peer_port > 65535) {
+                                	if (Logger.isEnabled())
+                                		Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
+                                				"Invalid peer port given: " + ip + ": " + peer_port));
+                                	continue;
+                                }
 								
 								byte[] peer_peer_id;
 								
@@ -2183,19 +2178,34 @@ TRTrackerBTAnnouncerImpl
 									peer_peer_id = (byte[])s_peerid ; 
 								}
 								
-								short protocol = DownloadAnnounceResultPeer.PROTOCOL_NORMAL;
+								short 	protocol;
 								
+								if ( crypto_flags == null ){
+									
+									protocol 	= DownloadAnnounceResultPeer.PROTOCOL_NORMAL;
+									
+								}else{
+									
+									protocol = crypto_flags[i]==0?DownloadAnnounceResultPeer.PROTOCOL_NORMAL:DownloadAnnounceResultPeer.PROTOCOL_CRYPT;
+								}
+								
+								int		udp_port	= 0;
+								
+								TRTrackerAnnouncerResponsePeerImpl new_peer = 
+									new TRTrackerAnnouncerResponsePeerImpl( 
+										PEPeerSource.PS_BT_TRACKER, 
+										peer_peer_id, 
+										ip, 
+										peer_port,
+										udp_port,
+										protocol  );
+										
 								if (Logger.isEnabled())
 									Logger.log(new LogEvent(torrent, LOGID,
-											"NON-COMPACT PEER: ip=" + ip + ",port=" + peer_port + ",prot=" + protocol));
+											"NON-COMPACT PEER: " + new_peer.getString()));
+
+								valid_meta_peers.add( new_peer );
 
-								valid_meta_peers.add(
-									new TRTrackerAnnouncerResponsePeerImpl( 
-											PEPeerSource.PS_BT_TRACKER, 
-											peer_peer_id, 
-											ip, 
-											peer_port,
-											protocol  ));
 								
 							} 
 						}
@@ -2205,8 +2215,25 @@ TRTrackerBTAnnouncerImpl
 				    	
 				    
 				    	byte[]	meta_peers = (byte[])meta_peers_peek;
+				    					    	
+				    	boolean	az_compact = metaData.get( "azcompact" ) != null;
 				    	
-				    	for (int i=0;i<meta_peers.length;i+=6){
+				    	int	entry_size = az_compact?9:6;
+				    	
+				    	if ( crypto_flags != null && meta_peers.length/entry_size != crypto_flags.length ){
+							
+							crypto_flags = null;
+							
+                           	if (Logger.isEnabled())
+                        		Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
+                        				"Invalid crypto_flags returned: length mismatch" ));
+						}
+
+				    	int	peer_number = 0;
+				    	
+				    	for (int i=0;i<meta_peers.length;i+=entry_size){
+				    		
+				    		peer_number++;
 				    		
 				    		int	ip1 = 0xFF & meta_peers[i];
 				    		int	ip2 = 0xFF & meta_peers[i+1];
@@ -2215,40 +2242,69 @@ TRTrackerBTAnnouncerImpl
 				    		int	po1 = 0xFF & meta_peers[i+4];
 				    		int	po2 = 0xFF & meta_peers[i+5];
 				    		
-				    		String	ip 		= "" + ip1 + "." + ip2 + "." + ip3 + "." + ip4;
-				    		int		peer_port 	= po1*256+po2;
+				    		String	ip 			= "" + ip1 + "." + ip2 + "." + ip3 + "." + ip4;
+				    		int		tcp_port 	= po1*256+po2;
 				    		
-                if (peer_port < 0 || peer_port > 65535) {
-                	if (Logger.isEnabled())
-										Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
-												"Invalid compact peer port given: " + ip + ": "
-														+ peer_port));
-                  continue;
-                }
+				    		if (tcp_port < 0 || tcp_port > 65535) {
+				    			if (Logger.isEnabled())
+				    				Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
+				    						"Invalid compact peer port given: " + ip + ": "
+				    						+ tcp_port));
+				    			continue;
+				    		}
                 
-				    		byte[]	peer_peer_id = getAnonymousPeerId( ip, peer_port );
+				    		byte[]	peer_peer_id = getAnonymousPeerId( ip, tcp_port );
 							
-				    		short protocol = DownloadAnnounceResultPeer.PROTOCOL_NORMAL;
+				    		short 	protocol;
+				    		int		udp_port;
 				    		
-				    		if (Logger.isEnabled())
-									Logger.log(new LogEvent(torrent, LOGID, "COMPACT PEER: ip="
-											+ ip + ",port=" + peer_port + ",prot=" + protocol ));
+				    		if ( az_compact ){
+				    			
+					    		int	upo1 = 0xFF & meta_peers[i+6];
+					    		int	upo2 = 0xFF & meta_peers[i+7];
+					    		
+					    		udp_port 	= upo1*256+upo2;
 
-				    		valid_meta_peers.add(
+					    		byte	flags = meta_peers[i+8];
+					    		
+					    		protocol = (flags&0x01)==0?DownloadAnnounceResultPeer.PROTOCOL_NORMAL:DownloadAnnounceResultPeer.PROTOCOL_CRYPT;
+					    		
+				    		}else{
+				    			
+				    			
+								if ( crypto_flags == null ){
+									
+					    			protocol 	= DownloadAnnounceResultPeer.PROTOCOL_NORMAL;
+
+								}else{
+									
+									protocol = crypto_flags[peer_number-1]==0?DownloadAnnounceResultPeer.PROTOCOL_NORMAL:DownloadAnnounceResultPeer.PROTOCOL_CRYPT;
+								}
+								
+				    			udp_port	= 0;
+				    		}		    		
+
+				    		TRTrackerAnnouncerResponsePeerImpl peer = 
 				    			new TRTrackerAnnouncerResponsePeerImpl( 
-				    					PEPeerSource.PS_BT_TRACKER, 
-				    					peer_peer_id, 
-				    					ip, 
-				    					peer_port,
-				    					protocol ));
-                			
+			    					PEPeerSource.PS_BT_TRACKER, 
+			    					peer_peer_id, 
+			    					ip, 
+			    					tcp_port,
+			    					udp_port,
+			    					protocol );
+			    			
+				    		if (Logger.isEnabled())
+								Logger.log(new LogEvent(torrent, LOGID, "COMPACT PEER: " + peer.getString()));
+
+				    		valid_meta_peers.add( peer );
 				    	}
 				    }else{
 						
 						throw( new IOException( "peers missing from response" ));
 				    }
 				    
-					TRTrackerAnnouncerResponsePeer[] peers=new TRTrackerAnnouncerResponsePeer[valid_meta_peers.size()];
+				    
+					TRTrackerAnnouncerResponsePeerImpl[] peers=new TRTrackerAnnouncerResponsePeerImpl[valid_meta_peers.size()];
 					
 					valid_meta_peers.toArray(peers);
 					
@@ -2296,25 +2352,37 @@ TRTrackerBTAnnouncerImpl
 
 						int incomplete = incomplete_l == null ? 0 : incomplete_l.intValue();
 
-						TRTrackerScraper scraper = TRTrackerScraperFactory.getSingleton();
+						if (complete < 0 || incomplete < 0) {
+							resp.setFailurReason(MessageText.getString(
+									"Tracker.announce.ignorePeerSeed",
+									new String[] { (complete < 0
+											? MessageText.getString("MyTorrentsView.seeds") + " == "
+													+ complete + ". " : "")
+											+ (incomplete < 0
+													? MessageText.getString("MyTorrentsView.peers")
+															+ " == " + incomplete + ". " : "") }));
+						} else {
 
-						if (scraper != null) {
-							TRTrackerScraperResponse scrapeResponse = scraper.scrape(this);
-							if (scrapeResponse != null) {
-								long lNextScrapeTime = scrapeResponse.getNextScrapeStartTime();
-								long lNewNextScrapeTime = TRTrackerScraperResponseImpl
-										.calcScrapeIntervalSecs(0, complete) * 1000;
+							TRTrackerScraper scraper = TRTrackerScraperFactory.getSingleton();
 
-								// make it look as if the scrape has just run. Important
-								// as seeding rules may make calculations on when the 
-								// scrape value were set
+							if (scraper != null) {
+								TRTrackerScraperResponse scrapeResponse = scraper.scrape(this);
+								if (scrapeResponse != null) {
+									long lNextScrapeTime = scrapeResponse.getNextScrapeStartTime();
+									long lNewNextScrapeTime = TRTrackerScraperResponseImpl.calcScrapeIntervalSecs(
+											0, complete) * 1000;
 
-								scrapeResponse.setScrapeStartTime(SystemTime.getCurrentTime());
+									// make it look as if the scrape has just run. Important
+									// as seeding rules may make calculations on when the 
+									// scrape value were set
 
-								if (lNextScrapeTime < lNewNextScrapeTime)
-									scrapeResponse.setNextScrapeStartTime(lNewNextScrapeTime);
+									scrapeResponse.setScrapeStartTime(SystemTime.getCurrentTime());
 
-								scrapeResponse.setSeedsPeers(complete, incomplete);
+									if (lNextScrapeTime < lNewNextScrapeTime)
+										scrapeResponse.setNextScrapeStartTime(lNewNextScrapeTime);
+
+									scrapeResponse.setSeedsPeers(complete, incomplete);
+								}
 							}
 						}
 					}
@@ -2381,8 +2449,6 @@ TRTrackerBTAnnouncerImpl
 	destroy()
 	{       
 		destroyed	= true;
-
-		COConfigurationManager.removeParameterListener("TCP.Announce.Port",this);
 		
 		TRTrackerAnnouncerFactoryImpl.destroy( this );
 		
@@ -2499,22 +2565,23 @@ TRTrackerBTAnnouncerImpl
 		}else{
 			DownloadAnnounceResultPeer[]	ext_peers = result.getPeers();
 			
-			TRTrackerAnnouncerResponsePeer[] peers = new TRTrackerAnnouncerResponsePeer[ext_peers.length];
+			TRTrackerAnnouncerResponsePeerImpl[] peers = new TRTrackerAnnouncerResponsePeerImpl[ext_peers.length];
 				
 			for (int i=0;i<ext_peers.length;i++){
 				
 				DownloadAnnounceResultPeer	ext_peer = ext_peers[i];
 				
-				if (Logger.isEnabled())
-					Logger.log(new LogEvent(torrent, LOGID, "EXTERNAL PEER: ip="
-							+ ext_peer.getAddress() + ",port=" + ext_peer.getPort()+",prot=" + ext_peer.getProtocol()));
-
 				peers[i] = new TRTrackerAnnouncerResponsePeerImpl( 
 								ext_peer.getSource(),
 								ext_peer.getPeerID(),
 								ext_peer.getAddress(), 
 								ext_peer.getPort(),
+								ext_peer.getUDPPort(),
 								ext_peer.getProtocol());
+				
+				if (Logger.isEnabled())
+					Logger.log(new LogEvent(torrent, LOGID, "EXTERNAL PEER: " + peers[i].getString())); 
+
 			}
 			
 			addToTrackerCache( peers);
@@ -2535,11 +2602,4 @@ TRTrackerBTAnnouncerImpl
 		
 		listeners.dispatch( LDT_TRACKER_RESPONSE, response );
 	}
-  
-  // ParameterListener Implementation
-  public void parameterChanged(String parameterName) {
-    if("TCP.Announce.Port".equals(parameterName)) {
-      setPort();
-    }
-  }
 }
\ No newline at end of file
