@@ -96,6 +96,22 @@ TRHostImpl
 			}
 		});	
 	
+	private static boolean host_add_announce_urls;
+	
+	static{
+		COConfigurationManager.addAndFireParameterListener(
+				"Tracker Host Add Our Announce URLs",
+				new ParameterListener()
+				{
+					public void 
+					parameterChanged(
+						String name )
+					{
+						host_add_announce_urls = COConfigurationManager.getBooleanParameter( name );
+					}
+				});
+	}
+	
 	private List	auth_listeners		= new ArrayList();
 	
 	private boolean	server_factory_listener_added;
@@ -144,6 +160,8 @@ TRHostImpl
 						{
 							private int	tick_count = 0;
 							
+							private Set	failed_ports = new HashSet();
+							
 							public void
 							runSupport()
 							{
@@ -190,7 +208,16 @@ TRHostImpl
 													
 												}catch( Throwable e ){
 											
-													Debug.printStackTrace( e );
+													Integer port_i = new Integer(port);
+													
+													if ( !failed_ports.contains(port_i)){
+														
+														failed_ports.add( port_i );
+														
+														Logger.log(
+																new LogEvent(LOGID,
+																"Tracker Host: failed to start server", e));
+													}
 												}
 											}
 										}
@@ -312,7 +339,10 @@ TRHostImpl
 			
 			if ( persistent && state != TRHostTorrent.TS_PUBLISHED ){
 
-				addTrackerAnnounce( torrent );
+				if ( host_add_announce_urls ){
+					
+					addTrackerAnnounce( torrent );
+				}
 			}
 			
 			TRHostTorrent	ht = lookupHostTorrent( torrent );
@@ -521,11 +551,8 @@ TRHostImpl
 					server.addListener( this );
 							
 				}catch( TRTrackerServerException e ){
-						
-					Logger.log(new LogEvent(LOGID,
-							"Tracker Host: failed to start server", e));
-		
-					throw( new TRHostException( e.getMessage()));
+								
+					throw( new TRHostException( "startServer failed", e ));
 				}
 			}
 			
@@ -591,7 +618,22 @@ TRHostImpl
 			// set the ip override so that we announce ourselves to other peers via the 
 			// real external address, not the local one used to connect to the tracker 
 			
-		tracker_client.setIPOverride( torrent.getAnnounceURL().getHost());
+		URL	announce = torrent.getAnnounceURL();
+		
+		if ( host_add_announce_urls ){
+		
+			tracker_client.setIPOverride( announce.getHost());
+			
+		}else{
+			
+				// prolly a backup tracker, we only want to override the IP if we're hosting it
+			
+			if ( TRTrackerUtils.isHosting( announce )){
+				
+				tracker_client.setIPOverride( announce.getHost());
+
+			}
+		}
 		
 			// hook into the client so that when the announce succeeds after the refresh below
 			// we can force a rescrape to pick up the new status 
@@ -944,13 +986,13 @@ TRHostImpl
 	
 	public boolean
 	handleExternalRequest(
-		String			client_address,
-		String			user,
-		String			url,
-		URL				absolute_url,
-		String			header,
-		InputStream		is,
-		OutputStream	os )
+		InetSocketAddress	client_address,
+		String				user,
+		String				url,
+		URL					absolute_url,
+		String				header,
+		InputStream			is,
+		OutputStream		os )
 		
 		throws IOException
 	{
@@ -1056,6 +1098,10 @@ TRHostImpl
 					}
 				}catch( TRHostException e ){
 					
+					throw( new TRTrackerServerException( e.getMessage(), e ));
+					
+				}catch( Throwable e ){
+					
 					throw( new TRTrackerServerException( "Pre-process fails", e ));
 				}
 			}
