@@ -23,7 +23,7 @@ package org.gudy.azureus2.pluginsimpl.local;
 
 import java.util.*;
 import java.io.File;
-import java.net.URL;
+import java.net.URL; 
 
 import org.gudy.azureus2.platform.PlatformManagerFactory;
 import org.gudy.azureus2.plugins.*;
@@ -34,6 +34,7 @@ import org.gudy.azureus2.pluginsimpl.local.clientid.ClientIDManagerImpl;
 import org.gudy.azureus2.pluginsimpl.local.ddb.DDBaseImpl;
 import org.gudy.azureus2.pluginsimpl.local.download.DownloadManagerImpl;
 import org.gudy.azureus2.pluginsimpl.local.installer.PluginInstallerImpl;
+import org.gudy.azureus2.pluginsimpl.local.ipc.IPCInterfaceImpl;
 import org.gudy.azureus2.pluginsimpl.local.ipfilter.IPFilterImpl;
 import org.gudy.azureus2.pluginsimpl.local.logging.LoggerImpl;
 import org.gudy.azureus2.pluginsimpl.local.messaging.MessageManagerImpl;
@@ -48,6 +49,7 @@ import org.gudy.azureus2.pluginsimpl.local.ui.config.ParameterRepository;
 import org.gudy.azureus2.pluginsimpl.local.ui.config.PluginConfigUIFactoryImpl;
 import org.gudy.azureus2.pluginsimpl.local.utils.*;
 import org.gudy.azureus2.pluginsimpl.local.update.*;
+import org.gudy.azureus2.plugins.ipc.IPCInterface;
 import org.gudy.azureus2.plugins.ipfilter.IPFilter;
 import org.gudy.azureus2.plugins.tracker.Tracker;
 import org.gudy.azureus2.plugins.ui.config.Parameter;
@@ -97,7 +99,7 @@ PluginInterfaceImpl
   private boolean				operational;
   private boolean				disabled;
   private Logger				logger;
-   
+  private IPCInterface			ipc_interface;
   private List					children		= new ArrayList();
   
   public 
@@ -123,6 +125,7 @@ PluginInterfaceImpl
     config 				= new PluginConfigImpl(this,pluginConfigKey);
     plugin_id			= _plugin_id;
     plugin_version		= _plugin_version;
+    ipc_interface		= new IPCInterfaceImpl( plugin );
   }
   
   	public Plugin
@@ -391,7 +394,7 @@ PluginInterfaceImpl
   public Utilities
   getUtilities()
   {
-  	return( new UtilitiesImpl( this ));
+  	return( new UtilitiesImpl( initialiser.getAzureusCore(), this ));
   }
   
   public ShortCuts
@@ -514,6 +517,10 @@ PluginInterfaceImpl
   		
   		((PluginInterface)children.get(i)).unload();
   	}
+  	
+  	setOperational(false);
+
+  	class_loader = null;
   }
   
   public void
@@ -546,7 +553,7 @@ PluginInterfaceImpl
    }
    
    public MessageManager getMessageManager() {
-     return MessageManagerImpl.getSingleton();
+     return MessageManagerImpl.getSingleton( initialiser.getAzureusCore() );
    }
    
    
@@ -575,6 +582,11 @@ PluginInterfaceImpl
   			Debug.printStackTrace( e );
   		}
   	}
+  	
+  	for (int i=0;i<children.size();i++){
+  		
+  		((PluginInterfaceImpl)children.get(i)).initialisationComplete();
+  	}
   }
   
   protected void
@@ -590,6 +602,11 @@ PluginInterfaceImpl
   			Debug.printStackTrace( e );
   		}
   	}
+  	
+  	for (int i=0;i<children.size();i++){
+  		
+  		((PluginInterfaceImpl)children.get(i)).closedownInitiated();
+  	}
   }
   
   protected void
@@ -605,11 +622,16 @@ PluginInterfaceImpl
   			Debug.printStackTrace( e );
   		}
   	}
+  	
+ 	for (int i=0;i<children.size();i++){
+  		
+  		((PluginInterfaceImpl)children.get(i)).closedownComplete();
+  	}
   }
   
-  protected void
-  fireEvent(
-  	PluginEvent		event )
+  public void
+  firePluginEvent(
+	PluginEvent		event )
   {
   	for (int i=0;i<event_listeners.size();i++){
   		
@@ -620,7 +642,12 @@ PluginInterfaceImpl
   			
   			Debug.printStackTrace( e );
   		}
-  	} 	
+  	} 
+  	
+	for (int i=0;i<children.size();i++){
+  		
+  		((PluginInterfaceImpl)children.get(i)).firePluginEvent(event);
+  	}
   }
   
   public ClassLoader
@@ -668,6 +695,12 @@ PluginInterfaceImpl
 		}
 	}
 	
+	 public IPCInterface 
+	 getIPC() 
+	 {
+		 return( ipc_interface );
+	 }
+	 
   public void
   addListener(
   	PluginListener	l )
