@@ -21,8 +21,6 @@
  
 package org.gudy.azureus2.ui.swt;
 
-import java.net.MalformedURLException;
-import java.net.URL;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.StringTokenizer;
@@ -33,37 +31,48 @@ import org.eclipse.swt.custom.ScrolledComposite;
 import org.eclipse.swt.dnd.*;
 import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
+import org.eclipse.swt.layout.FormAttachment;
+import org.eclipse.swt.layout.FormData;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.program.Program;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.UrlUtils;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.mainwindow.SWTThread;
 import org.gudy.azureus2.ui.swt.mainwindow.TorrentOpener;
 import org.gudy.azureus2.ui.swt.views.utils.VerticalAligner;
 
 import com.aelitis.azureus.core.impl.AzureusCoreImpl;
+import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
+import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
 
 /**
  * @author Olivier
  * 
  */
 public class Utils {
+  public static final boolean isGTK	= SWT.getPlatform().equals("gtk");
+
 	/** Some platforms expand the last column to fit the remaining width of
 	 * the table.
 	 */
-	public static final boolean LAST_TABLECOLUMN_EXPANDS = Constants.isLinux;
-
-  public static final boolean isGTK	= SWT.getPlatform().equals("gtk");
+	public static final boolean LAST_TABLECOLUMN_EXPANDS = isGTK;
+	
+	/** GTK already handles alternating background for tables */
+	public static final boolean TABLE_GRIDLINE_IS_ALTERNATING_COLOR = isGTK;
 
 	private static final boolean DIRECT_SETCHECKED = !Constants.isOSX
 			|| SWT.getVersion() >= 3212;
-  
+
+	public static final boolean SWT32_TABLEPAINT = false; //SWT.getVersion() >= 3200;
+			
   public static void disposeComposite(Composite composite,boolean disposeSelf) {
     if(composite == null || composite.isDisposed())
       return;
@@ -100,10 +109,15 @@ public class Utils {
    * @param disposeList
    */
   public static void disposeSWTObjects(List disposeList) {
+  	disposeSWTObjects(disposeList.toArray());
+		disposeList.clear();
+  }
+
+  public static void disposeSWTObjects(Object[] disposeList) {
   	boolean bResourceObjectExists = SWT.getVersion() >= 3129;
   	
-		for (int i = 0; i < disposeList.size(); i++) {
-			Object o = disposeList.get(i);
+		for (int i = 0; i < disposeList.length; i++) {
+			Object o = disposeList[i];
 			if (o instanceof Widget && !((Widget) o).isDisposed())
 				((Widget) o).dispose();
 			else if (bResourceObjectExists && (o instanceof Resource)
@@ -132,7 +146,6 @@ public class Utils {
 				// Resource
 			}
 		}
-		disposeList.clear();
   }
   
   /**
@@ -140,8 +153,8 @@ public class Utils {
    * If a valid link is found in the clipboard, it will be inserted
    * and the size (and location) of the dialog is adjusted.
    * @param shell to set the dialog location if needed
-   * @param gridData to adjust the dialog with
    * @param url the URL text control
+   * @param accept_magnets 
    *
    * @author Rene Leonhardt
    */
@@ -157,6 +170,7 @@ public class Utils {
    * <p>Gets an URL from the clipboard if a valid URL for downloading has been copied.</p>
    * <p>The supported protocols currently are http, https, and magnet.</p>
    * @param display
+   * @param accept_magnets 
    * @return first valid link from clipboard, else "http://"
    */
   public static String 
@@ -164,71 +178,40 @@ public class Utils {
 	 Display 	display,
 	 boolean	accept_magnets ) 
   {
-    final String[] prefixes = new String[] {"http://", "https://", "magnet:?", "magnet://?" };
-    
     final Clipboard cb = new Clipboard(display);
     final TextTransfer transfer = TextTransfer.getInstance();
     
     String data = (String)cb.getContents(transfer);
     
-    if (data != null) {
-      data	= data.trim();
-      for(int i = 0; i < (accept_magnets?prefixes.length:2 ); i++) {
-        final int begin = data.indexOf(prefixes[i]);
-        if (begin >= 0) {
-          final int end = data.indexOf("\n", begin + prefixes[i].length());
-          final String stringURL = (end >= 0) ? data.substring(begin, end - 1) : data.substring(begin);
-          try {
-            final URL parsedURL = new URL(stringURL);
-            return parsedURL.toExternalForm();
-          } catch (MalformedURLException e1) {
-          }
-        }
-      }
-    
-      if ( accept_magnets && data.length() == 40 ){
-    	  
-    	  for (int i=0;i<data.length();i++){
-    		  
-    		  if ( "0123456789abcdefABCDEF".indexOf( data.charAt(i)) == -1 ){
-    			  
-    			  return( prefixes[0] );
-    		  }
-    	  }
-    	  
-    	  	// accept raw hash of 40 hex chars
-    	  
-    	  return( data );
-      }
+    String text = UrlUtils.parseTextForURL(data, accept_magnets);
+    if (text == null) {
+    	return "http://";
     }
     
-    return prefixes[0];
+    return text;
   }
 
   public static void centreWindow(Shell shell) {
-		Rectangle displayBounds; // whole display area (including taskbar)
-		Rectangle displayClientArea; // area to center in
+		Rectangle displayArea; // area to center in
 		try {
-			displayBounds = shell.getMonitor().getBounds();
-			displayClientArea = shell.getMonitor().getClientArea();
+			displayArea = shell.getMonitor().getClientArea();
 		} catch (NoSuchMethodError e) {
-			displayBounds = shell.getDisplay().getBounds();
-			displayClientArea = shell.getDisplay().getClientArea();
+			displayArea = shell.getDisplay().getClientArea();
 		}
 
 		Rectangle shellRect = shell.getBounds();
 
-		if (shellRect.height > displayBounds.height - 50) {
-			shellRect.height = displayBounds.height - 50;
-			displayClientArea.height = displayBounds.height;
+		if (shellRect.height > displayArea.height) {
+			shellRect.height = displayArea.height;
 		}
-		if (shellRect.width > displayBounds.width - 50) {
-			shellRect.width = displayBounds.width - 50;
-			displayClientArea.width = displayBounds.width;
+		if (shellRect.width > displayArea.width - 50) {
+			shellRect.width = displayArea.width;
 		}
 
-		shellRect.x = (displayClientArea.width - shellRect.width) / 2;
-		shellRect.y = (displayClientArea.height - shellRect.height) / 2;
+		shellRect.x = displayArea.x
+				+ (displayArea.width - shellRect.width) / 2;
+		shellRect.y = displayArea.y
+				+ (displayArea.height - shellRect.height) / 2;
 
 		shell.setBounds(shellRect);
 	}
@@ -248,14 +231,13 @@ public class Utils {
       );
   }
 
-  public static DropTarget createTorrentDropTarget(Control control,
+  public static void createTorrentDropTarget(Composite composite,
 			boolean bAllowShareAdd) {
   	try {
-  		return createDropTarget(control, bAllowShareAdd, null);
+  		createDropTarget(composite, bAllowShareAdd, null);
   	} catch (Exception e) {
       Debug.out(e);
   	}
-    return null;
 	}
 
 	/**
@@ -264,71 +246,124 @@ public class Utils {
 	 *
 	 * @author Rene Leonhardt
 	 */
-	public static DropTarget createURLDropTarget(final Control control,
-			final Text url) {
+	public static void createURLDropTarget(Composite composite,
+			Text url) {
 		try {
-			return createDropTarget(control, false, url);
+			createDropTarget(composite, false, url);
 		} catch (Exception e) {
 			Debug.out(e);
 		}
-		return null;
 	}
 
-	private static DropTarget createDropTarget(final Control control,
-			final boolean bAllowShareAdd, final Text url) {
-  	DropTarget dropTarget = new DropTarget(control, DND.DROP_DEFAULT
-				| DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK | DND.DROP_TARGET_MOVE);
-
+	private static void createDropTarget(Composite composite,
+			final boolean bAllowShareAdd, final Text url,
+			DropTargetListener dropTargetListener) {
+		
+		Transfer[] transferList;
   	if (SWT.getVersion() >= 3107) {
-			dropTarget.setTransfer(new Transfer[] { HTMLTransfer.getInstance(),
+  		transferList = new Transfer[] { HTMLTransfer.getInstance(),
 					URLTransfer.getInstance(), FileTransfer.getInstance(),
-					TextTransfer.getInstance() });
+					TextTransfer.getInstance() };
 		} else {
-			dropTarget.setTransfer(new Transfer[] { URLTransfer.getInstance(),
-					FileTransfer.getInstance(), TextTransfer.getInstance() });
+			transferList = new Transfer[] { URLTransfer.getInstance(),
+					FileTransfer.getInstance(), TextTransfer.getInstance() };
 		}
 
-  	dropTarget.addDropListener(new DropTargetAdapter() {
-			public void dropAccept(DropTargetEvent event) {
-				event.currentDataType = URLTransfer.pickBestType(event.dataTypes,
-						event.currentDataType);
-			}
-
-			public void dragOver(DropTargetEvent event) {
-				// skip setting detail if user is forcing a drop type (ex. via the
-				// ctrl key), providing that the operation is valid
-				if (event.detail != DND.DROP_DEFAULT
-						&& ((event.operations & event.detail) > 0))
-					return;
-
-				if ((event.operations & DND.DROP_LINK) > 0)
-					event.detail = DND.DROP_LINK;
-				else if ((event.operations & DND.DROP_DEFAULT) > 0)
-					event.detail = DND.DROP_DEFAULT;
-				else if ((event.operations & DND.DROP_COPY) > 0)
-					event.detail = DND.DROP_COPY;
-			}
-
-			public void drop(DropTargetEvent event) {
-				if (url == null || url.isDisposed()) {
-					TorrentOpener.openDroppedTorrents(AzureusCoreImpl.getSingleton(),
-							event, bAllowShareAdd);
-				} else {
-					if (event.data instanceof URLTransfer.URLType) {
-						if (((URLTransfer.URLType) event.data).linkURL != null)
-							url.setText(((URLTransfer.URLType) event.data).linkURL);
-					} else if (event.data instanceof String) {
-						String sURL = TorrentOpener.parseTextForURL((String) event.data);
-						if (sURL != null) {
-							url.setText(sURL);
-						}
-					}
-				}
+		
+		final DropTarget dropTarget = new DropTarget(composite, DND.DROP_DEFAULT
+				| DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK | DND.DROP_TARGET_MOVE);
+		dropTarget.setTransfer(transferList);
+		dropTarget.addDropListener(dropTargetListener);
+		composite.addListener(SWT.Dispose, new Listener() {
+			public void handleEvent(Event event) {
+				if (dropTarget != null && !dropTarget.isDisposed())
+					dropTarget.dispose();
 			}
 		});
 		
-		return dropTarget;
+		// On Windows, dropping on children moves up to parent
+		// On OSX, each child needs it's own drop.
+		if (Constants.isWindows || Constants.isOSX)
+			return;
+
+		Control[] children = composite.getChildren();
+		for (int i = 0; i < children.length; i++) {
+			Control control = children[i];
+			if (!control.isDisposed()) {
+				if (control instanceof Composite) {
+					createDropTarget((Composite) control, bAllowShareAdd, url,
+							dropTargetListener);
+				} else {
+					final DropTarget dropTarget2 = new DropTarget(control,
+							DND.DROP_DEFAULT | DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK
+									| DND.DROP_TARGET_MOVE);
+					dropTarget2.setTransfer(transferList);
+					dropTarget2.addDropListener(dropTargetListener);
+
+					control.addListener(SWT.Dispose, new Listener() {
+						public void handleEvent(Event event) {
+							if (dropTarget2 != null && !dropTarget2.isDisposed())
+								dropTarget2.dispose();
+						}
+					});
+				}
+			}
+		}
+	}
+
+	private static void createDropTarget(Composite composite,
+			boolean bAllowShareAdd, Text url) {
+		
+		URLDropTarget target = new URLDropTarget(url, bAllowShareAdd);
+		createDropTarget(composite, bAllowShareAdd, url, target);
   }
+	
+	private static class URLDropTarget extends DropTargetAdapter {
+		private final Text url;
+		private final boolean bAllowShareAdd;
+
+		public URLDropTarget(Text url, boolean bAllowShareAdd) {
+			this.url = url;
+			this.bAllowShareAdd = bAllowShareAdd;
+		}
+
+		public void dropAccept(DropTargetEvent event) {
+			event.currentDataType = URLTransfer.pickBestType(event.dataTypes,
+					event.currentDataType);
+		}
+
+		public void dragOver(DropTargetEvent event) {
+			// skip setting detail if user is forcing a drop type (ex. via the
+			// ctrl key), providing that the operation is valid
+			if (event.detail != DND.DROP_DEFAULT
+					&& ((event.operations & event.detail) > 0))
+				return;
+
+			if ((event.operations & DND.DROP_LINK) > 0)
+				event.detail = DND.DROP_LINK;
+			else if ((event.operations & DND.DROP_DEFAULT) > 0)
+				event.detail = DND.DROP_DEFAULT;
+			else if ((event.operations & DND.DROP_COPY) > 0)
+				event.detail = DND.DROP_COPY;
+		}
+
+		public void drop(DropTargetEvent event) {
+			if (url == null || url.isDisposed()) {
+				TorrentOpener.openDroppedTorrents(AzureusCoreImpl.getSingleton(),
+						event, bAllowShareAdd);
+			} else {
+				if (event.data instanceof URLTransfer.URLType) {
+					if (((URLTransfer.URLType) event.data).linkURL != null)
+						url.setText(((URLTransfer.URLType) event.data).linkURL);
+				} else if (event.data instanceof String) {
+					String sURL = UrlUtils.parseTextForURL((String) event.data, true);
+					if (sURL != null) {
+						url.setText(sURL);
+					}
+				}
+			}
+		}
+	}
 
   /**
    * Force label to use more vertical space if wrapped and in a GridLayout
@@ -412,8 +447,7 @@ public class Utils {
   } // class
 
   public static void alternateRowBackground(TableItem item) {
-  	// On linux, table lines are actually alternating background colors
-  	if (Constants.isLinux) {
+  	if (Utils.TABLE_GRIDLINE_IS_ALTERNATING_COLOR) {
   		if (!item.getParent().getLinesVisible())
   			item.getParent().setLinesVisible(true);
   		return;
@@ -433,8 +467,7 @@ public class Utils {
   	if (table == null || table.isDisposed())
   		return;
 
-  	// On linux, table lines are actually alternating background colors
-  	if (Constants.isLinux) {
+  	if (Utils.TABLE_GRIDLINE_IS_ALTERNATING_COLOR) {
   		if (!table.getLinesVisible())
   			table.setLinesVisible(true);
   		return;
@@ -532,24 +565,40 @@ public class Utils {
    */
   public static boolean execSWTThread(Runnable code,
 			boolean async) {
+  	
     SWTThread swt = SWTThread.getInstance();
     
+    Display display;
     if (swt == null) {
-    	System.err.println("SWT Thread not started yet");
-    	return false;
+    	display = Display.getDefault();
+    	if (display == null) {
+      	System.err.println("SWT Thread not started yet!");
+      	return false;
+    	}
+    } else {
+    	if (swt.isTerminated()) {
+    		return false;
+    	}
+      display = swt.getDisplay();
     }
     
-    Display display = swt.getDisplay();
 
-  	if (display == null || display.isDisposed())
+  	if (display == null || display.isDisposed() || code == null)
 			return false;
 
-		if (display.getThread() == Thread.currentThread())
+		if (display.getThread() == Thread.currentThread()) {
 			code.run();
-		else if (async)
-			display.asyncExec(code);
-		else
+		} else if (async) {
+			try {
+				display.asyncExec(code);
+			} catch (NullPointerException e) {
+				// If the display is being disposed of, asyncExec may give a null
+				// pointer error
+				return false;
+			}
+		} else {
 			display.syncExec(code);
+		}
 
 		return true;
 	}
@@ -624,16 +673,35 @@ public class Utils {
 	 * Bottom Index may be negative
 	 */ 
 	public static int getTableBottomIndex(Table table, int iTopIndex) {
-		// getItemHeight is slow on Linux, use getItem/getClientArea
+		// on Linux, getItemHeight is slow AND WRONG. so is getItem(x).getBounds().y 
 		// getItem(Point) is slow on OSX
-		
-		if (!table.isVisible())
+
+		int itemCount = table.getItemCount();
+		if (!table.isVisible() || iTopIndex >= itemCount)
 			return -1;
 		
-		if (Constants.isOSX)
-			return Math.min(iTopIndex
-					+ ((table.getClientArea().height - table.getHeaderHeight() - 1) / 
-							table.getItemHeight()), table.getItemCount() - 1);
+		if (Constants.isOSX) {
+			try {
+				TableItem item = table.getItem(iTopIndex);
+				Rectangle bounds = item.getBounds();
+				Rectangle clientArea = table.getClientArea();
+	
+				int itemHeight = table.getItemHeight();
+				int iBottomIndex = Math.min(iTopIndex
+						+ (clientArea.height + clientArea.y - bounds.y - 1) / itemHeight,
+						itemCount - 1);
+	
+	//			System.out.println(bounds + ";" + clientArea + ";" + itemHeight + ";bi="
+	//					+ iBottomIndex + ";ti=" + iTopIndex + ";"
+	//					+ (clientArea.height + clientArea.y - bounds.y - 1));
+				return iBottomIndex;
+			} catch (NoSuchMethodError e) {
+				// item.getBounds is 3.2
+				return Math.min(iTopIndex
+						+ ((table.getClientArea().height - table.getHeaderHeight() - 1) / 
+								table.getItemHeight()) + 1, table.getItemCount() - 1);
+			}
+		}
 
 		// getItem will return null if clientArea's height is smaller than
 		// header height.
@@ -645,7 +713,7 @@ public class Utils {
 		TableItem bottomItem = table.getItem(new Point(2,
 				table.getClientArea().height - 1));
   	int iBottomIndex = (bottomItem != null) ? table.indexOf(bottomItem) :
-			table.getItemCount() - 1;
+			itemCount - 1;
   	return iBottomIndex;
 	}
 	
@@ -703,7 +771,10 @@ public class Utils {
 						values[i - 1] = 0;
 				}
 				if (i == 4) {
-					shell.setBounds(values[0], values[1], values[2], values[3]);
+					Rectangle shellBounds = new Rectangle(values[0], values[1],
+							values[2], values[3]);
+					shell.setBounds(shellBounds);
+					verifyShellRect(shell, true);
 					bDidResize = true;
 				}
 			} catch (Exception e) {
@@ -764,5 +835,169 @@ public class Utils {
 				saveMetrics();
 		}
 	}
+
+	public static GridData setGridData(Composite composite, int gridStyle,
+			Control ctrlBestSize, int maxHeight) {
+		GridData gridData = new GridData(gridStyle);
+		gridData.heightHint = ctrlBestSize.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;
+		if (gridData.heightHint > maxHeight && maxHeight > 0)
+			gridData.heightHint = maxHeight;
+		composite.setLayoutData(gridData);
+
+		return gridData;
+	}
+	
+	public static FormData getFilledFormData() {
+		FormData formData = new FormData();
+		formData.top = new FormAttachment(0, 0);
+		formData.left = new FormAttachment(0, 0);
+		formData.right = new FormAttachment(100, 0);
+		formData.bottom = new FormAttachment(100, 0);
+		
+		return formData;
+	}
+	
+	public static int pixelsToPoint(int pixels, int dpi) {
+    return (pixels * 72) / dpi;
+	}
+
+	public static boolean drawImage(GC gc, Image image, Rectangle dstRect,
+			Rectangle clipping, int hOffset, int vOffset, boolean clearArea)
+	{
+		return drawImage(gc, image, new Point(0, 0), dstRect, clipping, hOffset,
+				vOffset, clearArea);
+	}
+
+	public static boolean drawImage(GC gc, Image image, Rectangle dstRect,
+			Rectangle clipping, int hOffset, int vOffset)
+	{
+		return drawImage(gc, image, new Point(0, 0), dstRect, clipping, hOffset,
+				vOffset, false);
+	}
+
+	public static boolean drawImage(GC gc, Image image, Point srcStart,
+			Rectangle dstRect, Rectangle clipping, int hOffset, int vOffset,
+			boolean clearArea)
+	{
+		Rectangle srcRect;
+		Point dstAdj;
+
+		if (clipping == null) {
+			dstAdj = new Point(0, 0);
+			srcRect = new Rectangle(srcStart.x, srcStart.y, dstRect.width,
+					dstRect.height);
+		} else {
+			if (!dstRect.intersects(clipping)) {
+				return false;
+			}
+
+			dstAdj = new Point(Math.max(0, clipping.x - dstRect.x), Math.max(0,
+					clipping.y - dstRect.y));
+
+			srcRect = new Rectangle(0, 0, 0, 0);
+			srcRect.x = srcStart.x + dstAdj.x;
+			srcRect.y = srcStart.y + dstAdj.y;
+			srcRect.width = Math.min(dstRect.width - dstAdj.x, clipping.x
+					+ clipping.width - dstRect.x);
+			srcRect.height = Math.min(dstRect.height - dstAdj.y, clipping.y
+					+ clipping.height - dstRect.y);
+		}
+
+		if (!srcRect.isEmpty()) {
+			try {
+				if (clearArea) {
+					gc.fillRectangle(dstRect.x + dstAdj.x + hOffset, dstRect.y + dstAdj.y
+							+ vOffset, srcRect.width, srcRect.height);
+				}
+				gc.drawImage(image, srcRect.x, srcRect.y, srcRect.width,
+						srcRect.height, dstRect.x + dstAdj.x + hOffset, dstRect.y
+								+ dstAdj.y + vOffset, srcRect.width, srcRect.height);
+			} catch (Exception e) {
+				System.out.println("drawImage: " + e.getMessage() + ": " + image + ", " + srcRect
+						+ ", " + (dstRect.x + dstAdj.y + hOffset) + ","
+						+ (dstRect.y + dstAdj.y + vOffset) + "," + srcRect.width + ","
+						+ srcRect.height + "; imageBounds = " + image.getBounds());
+			}
+		}
+
+		return true;
+	}
+
+	/**
+	 * @param area
+	 * @param event id
+	 * @param listener
+	 */
+	public static void addListenerAndChildren(Composite area, int event,
+			Listener listener)
+	{
+		area.addListener(event, listener);
+		Control[] children = area.getChildren();
+		for (int i = 0; i < children.length; i++) {
+			Control child = children[i];
+			if (child instanceof Composite) {
+				addListenerAndChildren((Composite)child, event, listener);
+			} else {
+				child.addListener(event, listener);
+			}
+		}
+	}
+	
+	public static Shell findAnyShell() {
+		// Pick the main shell if we can
+		UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
+		if (uiFunctions != null) {
+			Shell shell = uiFunctions.getMainShell();
+			if (shell != null && shell.isDisposed()) {
+				return shell;
+			}
+		}
+
+		// Get active shell from current display if we can
+		Display current = Display.getCurrent();
+		if (current == null) {
+			return null;
+		}
+		Shell shell = current.getActiveShell();
+		if (shell != null && !shell.isDisposed()) {
+			return shell;
+		}
+		
+		// Get first shell of current display if we can
+		Shell[] shells = current.getShells();
+		if (shells.length == 0) {
+			return null;
+		}
+		
+		if (shells[0] != null && !shells[0].isDisposed()) {
+			return shells[0];
+		}
+		
+		return null;
+	}
+
+	/**
+	 * @param listener
+	 */
+	public static boolean verifyShellRect(Shell shell, boolean bAdjustIfInvalid) {
+		boolean bMetricsOk;
+		try {
+			bMetricsOk = false;
+			Point ptTopLeft = shell.getLocation();
+
+			Monitor[] monitors = shell.getDisplay().getMonitors();
+			for (int j = 0; j < monitors.length && !bMetricsOk; j++) {
+				Rectangle bounds = monitors[j].getBounds();
+				bMetricsOk = bounds.contains(ptTopLeft);
+			}
+		} catch (NoSuchMethodError e) {
+			Rectangle bounds = shell.getDisplay().getBounds();
+			bMetricsOk = shell.getBounds().intersects(bounds);
+		}
+		if (!bMetricsOk && bAdjustIfInvalid) {
+			centreWindow(shell);
+		}
+		return bMetricsOk;
+	}
 }
 
