@@ -22,34 +22,37 @@ package org.gudy.azureus2.ui.swt.views;
 
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Comparator;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.SashForm;
 import org.eclipse.swt.custom.ScrolledComposite;
 import org.eclipse.swt.custom.StackLayout;
-import org.eclipse.swt.events.SelectionAdapter;
-import org.eclipse.swt.events.SelectionEvent;
+import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.FontData;
-import org.eclipse.swt.layout.FillLayout;
-import org.eclipse.swt.layout.GridData;
-import org.eclipse.swt.layout.GridLayout;
+import org.eclipse.swt.graphics.Image;
+import org.eclipse.swt.layout.*;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
-import org.gudy.azureus2.core3.logging.*;
-import org.gudy.azureus2.core3.util.AERunnable;
-import org.gudy.azureus2.plugins.ui.config.ConfigSection;
-import org.gudy.azureus2.plugins.ui.config.ConfigSectionSWT;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.pluginsimpl.local.ui.config.ConfigSectionRepository;
+import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.Utils;
-import org.gudy.azureus2.ui.swt.mainwindow.MainWindow;
 import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 import org.gudy.azureus2.ui.swt.views.configsections.*;
 
 import com.aelitis.azureus.core.AzureusCore;
 
+import org.gudy.azureus2.plugins.ui.config.ConfigSection;
+import org.gudy.azureus2.plugins.ui.config.ConfigSectionSWT;
+
 /**
  * @author Olivier
  *
@@ -65,10 +68,15 @@ public class ConfigView extends AbstractIView {
   StackLayout layoutConfigSection;
   Label lHeader;
   Font headerFont;
+  Font filterFoundFont;
   Tree tree;
   TreeItem treePlugins;
   ArrayList pluginSections;
 
+	private Timer filterDelayTimer;
+	private String filterText = "";
+	private Label lblX;
+
   /**
    * Main Initializer
    * 
@@ -87,24 +95,26 @@ public class ConfigView extends AbstractIView {
   public void initialize(Composite composite) {
     GridData gridData;
     /*
-    /--cConfig-------------------------------------------------------\
-    | ###SashForm#form############################################## |
-    | # /--tree--\ /--cRightSide---------------------------------\ # |
-    | # |        | | ***cHeader********************************* | # |
-    | # |        | | * lHeader                                 * | # |
-    | # |        | | ******************************************* | # |
-    | # |        | | ###Composite cConfigSection################ | # |
-    | # |        | | #                                         # | # |
-    | # |        | | #                                         # | # |
-    | # |        | | #                                         # | # |
-    | # |        | | #                                         # | # |
-    | # |        | | ########################################### | # |
-    | # \--------/ \---------------------------------------------/ # |
-    | ############################################################## |
-    |  [Button]                                                      |
-    \----------------------------------------------------------------/
+    /--cConfig-----------------------------------------------------------\
+    | ###SashForm#form################################################## |
+    | # /--cLeftSide-\ /--cRightSide---------------------------------\ # |
+    | # |txtFilter X | | ***cHeader********************************* | # |
+    | # | ##tree#### | | * lHeader                                 * | # |
+    | # | #        # | | ******************************************* | # |
+    | # | #        # | | ###Composite cConfigSection################ | # |
+    | # | #        # | | #                                         # | # |
+    | # | #        # | | #                                         # | # |
+    | # | #        # | | #                                         # | # |
+    | # | #        # | | #                                         # | # |
+    | # | ########## | | ########################################### | # |
+    | # \------------/ \---------------------------------------------/ # |
+    | ################################################################## |
+    |  [Button]                                                          |
+    \--------------------------------------------------------------------/
     */
     try {
+      Display d = composite.getDisplay();
+
       cConfig = new Composite(composite, SWT.NONE);
       GridLayout configLayout = new GridLayout();
       configLayout.marginHeight = 0;
@@ -116,9 +126,67 @@ public class ConfigView extends AbstractIView {
       SashForm form = new SashForm(cConfig,SWT.HORIZONTAL);
       gridData = new GridData(GridData.FILL_BOTH);
       form.setLayoutData(gridData);
+      
+      Composite cLeftSide = new Composite(form, SWT.BORDER);
+      gridData = new GridData(GridData.FILL_BOTH);
+      cLeftSide.setLayoutData(gridData);
+      
+      FormLayout layout = new FormLayout();
+      cLeftSide.setLayout(layout);
+      
+      final Text txtFilter = new Text(cLeftSide, SWT.BORDER);
+      final String sFilterText = MessageText.getString("ConfigView.filter");
+      txtFilter.setText(sFilterText);
+      txtFilter.selectAll();
+      txtFilter.addModifyListener(new ModifyListener() {
+      	public void modifyText(ModifyEvent e) {
+      		filterTree(txtFilter.getText());
+      	}
+      });
+      txtFilter.addMouseListener(new MouseAdapter() {
+				public void mouseDown(MouseEvent e) {
+					if (txtFilter.getText().equals(sFilterText)) {
+						txtFilter.selectAll();
+					}
+				}
+			});
+      txtFilter.setFocus();
+
+  		lblX = new Label(cLeftSide, SWT.WRAP);
+      Messages.setLanguageTooltip(lblX, "MyTorrentsView.clearFilter.tooltip");
+      lblX.setImage(ImageRepository.getImage("smallx-gray"));
+      lblX.addMouseListener(new MouseAdapter() {
+      	public void mouseUp(MouseEvent e) {
+      		txtFilter.setText("");
+      	}
+      });
   
-      tree = new Tree(form, SWT.BORDER);
-      tree.setLayout(new FillLayout());
+      tree = new Tree(cLeftSide, SWT.NONE);
+      FontData[] fontData = tree.getFont().getFontData();
+      fontData[0].setStyle(SWT.BOLD);
+      filterFoundFont = new Font(d, fontData);
+      
+      cLeftSide.setBackground(tree.getBackground());
+      lblX.setBackground(tree.getBackground());
+
+      FormData formData;
+      formData = new FormData();
+      formData.top = new FormAttachment(0,5);
+      formData.left = new FormAttachment(0,5);
+      formData.right = new FormAttachment(lblX, -3);
+      txtFilter.setLayoutData(formData);
+
+      formData = new FormData();
+      formData.top = new FormAttachment(0,5);
+      formData.right = new FormAttachment(100,-5);
+      lblX.setLayoutData(formData);
+
+      formData = new FormData();
+      formData.top = new FormAttachment(txtFilter,5);
+      formData.left = new FormAttachment(0,0);
+      formData.right = new FormAttachment(100,0);
+      formData.bottom = new FormAttachment(100,0);
+      tree.setLayoutData(formData);
   
       Composite cRightSide = new Composite(form, SWT.NULL);
       configLayout = new GridLayout();
@@ -135,14 +203,13 @@ public class ConfigView extends AbstractIView {
       gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_CENTER);
       cHeader.setLayoutData(gridData);
   
-      Display d = cRightSide.getDisplay();
       cHeader.setBackground(d.getSystemColor(SWT.COLOR_LIST_SELECTION));
       cHeader.setForeground(d.getSystemColor(SWT.COLOR_LIST_SELECTION_TEXT));
   
       lHeader = new Label(cHeader, SWT.NULL);
       lHeader.setBackground(d.getSystemColor(SWT.COLOR_LIST_SELECTION));
       lHeader.setForeground(d.getSystemColor(SWT.COLOR_LIST_SELECTION_TEXT));
-      FontData[] fontData = lHeader.getFont().getFontData();
+      fontData = lHeader.getFont().getFontData();
       fontData[0].setStyle(SWT.BOLD);
       int fontHeight = (int)(fontData[0].getHeight() * 1.2);
       fontData[0].setHeight(fontHeight);
@@ -199,9 +266,12 @@ public class ConfigView extends AbstractIView {
                                          new ConfigSectionConnectionAdvanced(),
                                          new ConfigSectionConnectionEncryption(),
                                          new ConfigSectionTransfer(),
+                                         new ConfigSectionTransferAutoSpeed(),
                                          new ConfigSectionTransferLAN(),
                                          new ConfigSectionFile(), 
+                                         new ConfigSectionFileMove(),
                                          new ConfigSectionFileTorrents(),
+                                         new ConfigSectionFileTorrentsDecoding(),
                                          new ConfigSectionFilePerformance(),
                                          new ConfigSectionInterface(),
                                          new ConfigSectionInterfaceLanguage(),
@@ -241,7 +311,24 @@ public class ConfigView extends AbstractIView {
 							+ "configSectionGetName function", e));
           name = "Bad Plugin";
         }
-        try {
+
+         String	section_key = name;
+         
+         if ( plugin_section ){
+         		// if resource exists without prefix then use it as plugins don't
+         		// need to start with the prefix
+         	
+         	if ( !MessageText.keyExists(section_key)){
+         		
+         		section_key = sSectionPrefix + name;
+         	}
+         	
+         }else{
+         	
+         	section_key = sSectionPrefix + name;
+         }
+         
+         try {
           TreeItem treeItem = null;
           String location = section.configSectionGetParentSection();
   
@@ -250,9 +337,23 @@ public class ConfigView extends AbstractIView {
           else if (location != "") {
             TreeItem treeItemFound = findTreeItem(tree, location);
             if (treeItemFound != null)
-              treeItem = new TreeItem(treeItemFound, SWT.NULL);
+              if (location.equalsIgnoreCase(ConfigSection.SECTION_PLUGINS)) {
+            	  // Force ordering by name here.
+            	  int position = findInsertPointFor(MessageText.getString(section_key), treeItemFound);
+            	  if (position == -1) {
+            		  treeItem = new TreeItem(treeItemFound, SWT.NULL);
+            	  }
+            	  else {
+            		  treeItem = new TreeItem(treeItemFound, SWT.NULL, position);
+            	  }
+              }
+              else {
+            	  treeItem = new TreeItem(treeItemFound, SWT.NULL);
+              }
           }
-  
+
+          // XXX: Is this statement ever executed? I can't see where
+          // treePlugins ever gets initialised...
           if (treeItem == null)
             treeItem = new TreeItem(treePlugins, SWT.NULL);
   
@@ -260,6 +361,13 @@ public class ConfigView extends AbstractIView {
           sc.setExpandHorizontal(true);
           sc.setExpandVertical(true);
           sc.setLayoutData(new GridData(GridData.FILL_BOTH));
+      		sc.getVerticalBar().setIncrement(16);
+      		sc.addListener(SWT.Resize, new Listener() {
+      			public void handleEvent(Event event) {
+      				ScrolledComposite sc = (ScrolledComposite)event.widget;
+      	  		sc.getVerticalBar().setPageIncrement(sc.getSize().y);
+      			}
+      		});
           
           if(i == 0) {
             Composite c;
@@ -273,21 +381,6 @@ public class ConfigView extends AbstractIView {
             }
             sc.setContent(c);
           }
-          String	section_key = name;
-          
-          if ( plugin_section ){
-          		// if resource exists without prefix then use it as plugins don't
-          		// need to start with the prefix
-          	
-          	if ( !MessageText.keyExists(section_key)){
-          		
-          		section_key = sSectionPrefix + name;
-          	}
-          	
-          }else{
-          	
-          	section_key = sSectionPrefix + name;
-          }
           
           Messages.setLanguageText(treeItem, section_key);
           treeItem.setData("Panel", sc);
@@ -316,7 +409,133 @@ public class ConfigView extends AbstractIView {
     showSection(items[0]);
   }
 
-  private void showSection(TreeItem section) {
+  /**
+	 * @param text
+	 */
+	protected void filterTree(String text) {
+		filterText = text;
+		if (filterDelayTimer != null) {
+			filterDelayTimer.destroy();
+		}
+		
+		if (lblX != null && !lblX.isDisposed()) {
+			Image img = ImageRepository.getImage(filterText.length() > 0 ? "smallx"
+					: "smallx-gray");
+
+			lblX.setImage(img);
+		}
+
+
+		filterDelayTimer = new Timer("Filter");
+		filterDelayTimer.addEvent(SystemTime.getCurrentTime() + 300,
+				new TimerEventPerformer() {
+					public void perform(TimerEvent event) {
+						filterDelayTimer.destroy();
+						filterDelayTimer = null;
+
+						Utils.execSWTThread(new AERunnable() {
+							public void runSupport() {
+								// TODO Auto-generated method stub
+								ArrayList foundItems = new ArrayList();
+								TreeItem[] items = tree.getItems();
+								try {
+									tree.setRedraw(false);
+									for (int i = 0; i < items.length; i++) {
+										items[i].setExpanded(false);
+									}
+
+									filterTree(items, filterText, foundItems);
+								} finally {
+									tree.setRedraw(true);
+								}
+							}
+						});
+					}
+				});
+	}
+
+	protected void filterTree(TreeItem[] items, String text, ArrayList foundItems)
+	{
+		text = text.toLowerCase();
+		for (int i = 0; i < items.length; i++) {
+			ensureSectionBuilt(items[i]);
+			ScrolledComposite composite = (ScrolledComposite) items[i].getData("Panel");
+
+			if (text.length() > 0
+					&& (items[i].getText().toLowerCase().indexOf(text) >= 0 || compositeHasText(
+							composite, text))) {
+				foundItems.add(items[i]);
+
+				ensureExpandedTo(items[i]);
+				items[i].setFont(filterFoundFont);
+			} else {
+				items[i].setFont(null);
+			}
+			filterTree(items[i].getItems(), text, foundItems);
+		}
+	}
+	
+	private void ensureExpandedTo(TreeItem item) {
+    TreeItem itemParent = item.getParentItem();
+  	if (itemParent != null) {
+  		itemParent.setExpanded(true);
+  		ensureExpandedTo(itemParent);
+  	}
+	}
+
+	/**
+	 * @param composite 
+	 * @param text
+	 * @return
+	 */
+	private boolean compositeHasText(Composite composite, String text) {
+		Control[] children = composite.getChildren();
+		
+		for (int i = 0; i < children.length; i++) {
+			Control child = children[i];
+			if (child instanceof Label) {
+				if (((Label)child).getText().toLowerCase().indexOf(text) >= 0) {
+					return true;
+				}
+			} else if (child instanceof Group) {
+				if (((Group)child).getText().toLowerCase().indexOf(text) >= 0) {
+					return true;
+				}
+			} else if (child instanceof Button) {
+				if (((Button)child).getText().toLowerCase().indexOf(text) >= 0) {
+					return true;
+				}
+			}
+
+			if (child instanceof Composite) {
+				if (compositeHasText((Composite) child, text)) {
+					return true;
+				}
+			}
+		}
+		
+		return false;
+	}
+
+	private void showSection(TreeItem section) {
+    ScrolledComposite item = (ScrolledComposite)section.getData("Panel");
+
+    if (item != null) {
+    	
+    	ensureSectionBuilt(section);
+    	
+      layoutConfigSection.topControl = item;
+      
+      Composite c = (Composite)item.getContent();
+      
+      item.setMinSize(c.computeSize(SWT.DEFAULT, SWT.DEFAULT));
+      cConfigSection.layout();
+      
+      updateHeader(section);
+    }
+  }
+	
+	private void ensureSectionBuilt(TreeItem section) {
     ScrolledComposite item = (ScrolledComposite)section.getData("Panel");
 
     if (item != null) {
@@ -343,19 +562,9 @@ public class ConfigView extends AbstractIView {
         }
         
         item.setContent(c);
-        
-        c.layout();
       }
-      layoutConfigSection.topControl = item;
-      
-      Composite c = (Composite)item.getContent();
-      
-      item.setMinSize(c.computeSize(SWT.DEFAULT, SWT.DEFAULT));
-      cConfigSection.layout();
-      
-      updateHeader(section);
     }
-  }
+	}
 
   private void updateHeader(TreeItem section) {
     if (section == null)
@@ -387,8 +596,21 @@ public class ConfigView extends AbstractIView {
     sc.setExpandHorizontal(true);
     sc.setExpandVertical(true);
     sc.setLayoutData(new GridData(GridData.FILL_BOTH));
+		sc.getVerticalBar().setIncrement(16);
+		sc.addListener(SWT.Resize, new Listener() {
+			public void handleEvent(Event event) {
+				ScrolledComposite sc = (ScrolledComposite)event.widget;
+	  		sc.getVerticalBar().setPageIncrement(sc.getSize().y);
+			}
+		});
 
     Composite cConfigSection = new Composite(sc, SWT.NULL);
+    
+    String section_key = ((bPrefix) ? sSectionPrefix : "") + sNameID;
+    
+    if (position == -2) { // Means "auto-order".
+    	position = findInsertPointFor(MessageText.getString(section_key), (treeItemParent == null) ? (Object)tree : (Object)treeItemParent);
+    }
 
     TreeItem treeItem;
     if (treeItemParent == null) {
@@ -402,13 +624,51 @@ public class ConfigView extends AbstractIView {
       else
         treeItem = new TreeItem(treeItemParent, SWT.NULL);
     }
-    Messages.setLanguageText(treeItem, ((bPrefix) ? sSectionPrefix : "") + sNameID);
+    Messages.setLanguageText(treeItem, section_key);
     treeItem.setData("Panel", sc);
     treeItem.setData("ID", sNameID);
 
     sc.setContent(cConfigSection);
     return cConfigSection;
   }
+  
+  private static Comparator insert_point_comparator = new Comparator() {
+	  
+	  private String asString(Object o) {
+		  if (o instanceof String) {
+			  return (String)o;
+		  }
+		  else if (o instanceof TreeItem) {
+			  return ((TreeItem)o).getText();
+		  }
+		  else {
+			  throw new ClassCastException("object is not String or TreeItem: " + o.getClass().getName());
+		  }
+	  }
+	  
+	  public int compare(Object o1, Object o2) {
+		  int result = String.CASE_INSENSITIVE_ORDER.compare(asString(o1), asString(o2));
+		  return result;
+	  }
+  };
+  
+  public static int findInsertPointFor(String name, Object structure) {
+	  TreeItem[] children = null;
+	  if (structure instanceof Tree) {
+	      children = ((Tree)structure).getItems();
+	  }
+	  else {
+		  children = ((TreeItem)structure).getItems();
+	  }
+	  if (children.length == 0) {return -1;}
+	  int result =  Arrays.binarySearch(children, name, insert_point_comparator);
+	  if (result > 0) {return result;}
+	  result = -(result+1);
+	  if (result == children.length) {
+		  result = -1;
+	  }
+	  return result;
+  }
 
   public TreeItem findTreeItem(String ID) {
   	return findTreeItem((Tree)null, ID);
@@ -483,7 +743,6 @@ public class ConfigView extends AbstractIView {
   }
 
   public void delete() {
-    MainWindow.getWindow().clearConfig();
     for (int i = 0; i < pluginSections.size(); i++)
       ((ConfigSection)pluginSections.get(i)).configSectionDelete();
     pluginSections.clear();
@@ -499,10 +758,9 @@ public class ConfigView extends AbstractIView {
     }
     Utils.disposeComposite(cConfig);
 
-  	if (headerFont != null && !headerFont.isDisposed()) {
-  		headerFont.dispose();
-  		headerFont = null;
-  	}
+  	Utils.disposeSWTObjects(new Object[] { headerFont, filterFoundFont });
+		headerFont = null;
+		filterFoundFont = null;
   }
 
   public String getFullTitle() {
