@@ -98,7 +98,7 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 				+ "opendetails");
 		new BooleanParameter(cDisplay, "Open Bar", false, LBLKEY_PREFIX + "openbar");
 
-		if (!Constants.isOSX) {
+		if (!Constants.isOSX || SWT.getVersion() >= 3300) {
 
 			BooleanParameter est = new BooleanParameter(cDisplay,
 					"Enable System Tray", true, KEY_PREFIX + "enabletray");
@@ -114,6 +114,35 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 					.getControls()));
 
 		}
+		
+        /**
+         * Default download / upload limits available in the UI.
+         */
+        Group limit_group = new Group(cDisplay, SWT.NULL);
+        Messages.setLanguageText(limit_group, LBLKEY_PREFIX + "set_ui_transfer_speeds");
+        layout = new GridLayout();
+        limit_group.setLayout(layout);
+        
+        Label limit_group_label = new Label(limit_group, SWT.WRAP);
+        Messages.setLanguageText(limit_group_label, LBLKEY_PREFIX + "set_ui_transfer_speeds.description");
+        
+        String[] limit_types = new String[] {"download", "upload"};
+        final String limit_type_prefix = "config.ui.speed.partitions.manual.";
+        for (int i=0; i<limit_types.length; i++) {
+        	final BooleanParameter bp = new BooleanParameter(limit_group, limit_type_prefix + limit_types[i] + ".enabled", false, LBLKEY_PREFIX + "set_ui_transfer_speeds.description." + limit_types[i]);
+        	final StringParameter sp = new StringParameter(limit_group, limit_type_prefix + limit_types[i] + ".values", "");
+        	IAdditionalActionPerformer iaap = new GenericActionPerformer(new Control[] {}) {
+        		public void performAction() {
+        			sp.getControl().setEnabled(bp.isSelected());	
+        		}
+        	};
+        	
+            gridData = new GridData();
+            gridData.widthHint = 150;
+            sp.setLayoutData(gridData);
+        	iaap.performAction();
+        	bp.setAdditionalActionPerformer(iaap);
+        }
 
 		new BooleanParameter(cDisplay, "Send Version Info", true, LBLKEY_PREFIX
 				+ "allowSendVersion");
@@ -368,6 +397,26 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 		cArea.setLayout(layout);
 		cArea.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
 
+		BooleanParameter popup_dl_completed = new BooleanParameter(cArea,
+				"Popup Download Finished", false, LBLKEY_PREFIX + "popupdownloadfinished");
+		gridData = new GridData();
+		gridData.horizontalSpan = 2;
+		popup_dl_completed.setLayoutData(gridData);
+		
+		BooleanParameter popup_file_completed = new BooleanParameter(cArea,
+				"Popup File Finished", false, LBLKEY_PREFIX + "popupfilefinished");
+		gridData = new GridData();
+		gridData.horizontalSpan = 2;
+		popup_file_completed.setLayoutData(gridData);
+
+		cArea = new Composite(cDisplay, SWT.NULL);
+		layout = new GridLayout();
+		layout.marginHeight = 0;
+		layout.marginWidth = 0;
+		layout.numColumns = 2;
+		cArea.setLayout(layout);
+		cArea.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+
 		if (!Constants.isOSX) {
 
 			BooleanParameter confirm = new BooleanParameter(cArea,
@@ -377,7 +426,17 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 			gridData.horizontalSpan = 2;
 			confirm.setLayoutData(gridData);
 		}
+		
+		cArea = new Composite(cDisplay, SWT.NULL);
+		layout = new GridLayout();
+		layout.marginHeight = 0;
+		layout.marginWidth = 0;
+		layout.numColumns = 2;
+		cArea.setLayout(layout);
+		cArea.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
 
+		// config torrent removal
+		
 		BooleanParameter confirm_removal = new BooleanParameter(cArea,
 				"confirm_torrent_removal", KEY_PREFIX + "confirm_torrent_removal");
 		gridData = new GridData();
