@@ -27,6 +27,11 @@ package com.aelitis.azureus.core.diskmanager.file;
  */
 
 import java.io.File;
+import java.util.Map;
+
+import org.gudy.azureus2.core3.torrent.TOTorrent;
+
+import com.aelitis.azureus.core.util.CaseSensitiveFileMap;
 
 public interface 
 FMFileManager 
@@ -34,7 +39,18 @@ FMFileManager
 	public FMFile
 	createFile(
 		FMFileOwner	owner,
-		File		file )
+		File		file,
+		int			type )
 	
 		throws FMFileManagerException;
+	
+	public void
+	setFileLinks(
+		TOTorrent				torrent,
+		CaseSensitiveFileMap	links );
+	
+	public File
+	getFileLink(
+		TOTorrent			torrent,
+		File				from_file );
 }
