@@ -1,7 +1,7 @@
 /*
  * Created on 08-Dec-2004
  * Created by Paul Gardner
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -30,10 +30,37 @@ package com.aelitis.azureus.core.proxy.socks;
 public interface 
 AESocksProxy 
 {
+	public static final String	PV_4		= "V4";
+	public static final String	PV_4a		= "V4a";
+	public static final String	PV_5		= "V5";
+	
 	public int
 	getPort();
 	
 	public AESocksProxyPlugableConnection
 	getDefaultPlugableConnection(
 		AESocksProxyConnection		basis );
+	
+		/**
+		 * Set the next SOCKS proxy in a chain - i.e. this socks proxy's default plugable connection
+		 * will connect onwards using this SOCKS proxy
+		 *  
+		 * @param host
+		 * @param port
+		 */
+	
+	public void
+	setNextSOCKSProxy(
+		String		host,
+		int			port,
+		String		proxy_version );
+	
+	public String
+	getNextSOCKSProxyHost();
+	
+	public int
+	getNextSOCKSProxyPort();
+	
+	public String
+	getNextSOCKSProxyVersion();
 }
