@@ -27,21 +27,35 @@ package org.gudy.azureus2.core3.global;
  */
 
 import org.gudy.azureus2.core3.global.impl.*;
-import com.aelitis.azureus.core.AzureusCoreListener;
+
 
 public class 
 GlobalManagerFactory 
 {
 	public static GlobalManager
 	create(
-       AzureusCoreListener 	listener)
+		GlobalMangerProgressListener 	operation )
 	{
-		return( new GlobalManagerImpl( listener));
+		return( new GlobalManagerImpl( operation, 0 ));
 	}
   
 	public static GlobalManager
 	create()
 	{
-		return( new GlobalManagerImpl( null ));
+		return( new GlobalManagerImpl( null, 0 ));
+	}
+
+	public static GlobalManager
+	create(
+		GlobalMangerProgressListener 	operation,
+		long existingTorrentLoadDelay)
+	{
+		return( new GlobalManagerImpl( operation, existingTorrentLoadDelay ));
+	}
+  
+	public static GlobalManager
+	create(long existingTorrentLoadDelay)
+	{
+		return( new GlobalManagerImpl( null, existingTorrentLoadDelay ));
 	}
 }
