@@ -33,21 +33,55 @@ import javax.net.ssl.*;
 
 import java.security.*;
 import java.security.cert.*;
+import java.security.cert.Certificate;
 
-import org.gudy.azureus2.core3.logging.LGLogger;
+import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.security.*;
 import org.gudy.azureus2.core3.util.*;
 
+import com.aelitis.azureus.core.util.CopyOnWriteList;
+
 public class 
 SESecurityManagerImpl 
 {
+	private static final LogIDs LOGID = LogIDs.NET; 
+
 	protected static SESecurityManagerImpl	singleton = new SESecurityManagerImpl();
 	
+	protected static String	KEYSTORE_TYPE;
+	
+	static{
+		String[]	types = { "JKS", "GKR" };
+		
+		for (int i=0;i<types.length;i++){
+			try{
+				KeyStore.getInstance( types[i] );
+				
+				KEYSTORE_TYPE	= types[i];
+				
+				break;
+				
+			}catch( Throwable e ){
+			}
+		}
+		
+		if ( KEYSTORE_TYPE == null ){
+			
+				// it'll fail later but we need to use something here
+			
+			KEYSTORE_TYPE	= "JKS";
+		}
+		
+		Logger.log( new LogEvent(LOGID, "Keystore type is " + KEYSTORE_TYPE ));
+
+	}
+	
 	protected String	keystore_name;
 	protected String	truststore_name;
 	
-	protected List	certificate_listeners 	= new ArrayList();
-	protected List	password_listeners 		= new ArrayList();
+	protected List				certificate_listeners 	= new ArrayList();
+	protected CopyOnWriteList	password_listeners 		= new CopyOnWriteList();
 	
 	protected Map	password_handlers		= new HashMap();
 	
@@ -83,13 +117,27 @@ SESecurityManagerImpl
 		
 		installAuthenticator();
 		
-		try{
-			Security.addProvider((java.security.Provider)
-				Class.forName("com.sun.net.ssl.internal.ssl.Provider").newInstance());
+	
+		String[]	providers = { "com.sun.net.ssl.internal.ssl.Provider", "org.metastatic.jessie.provider.Jessie" };
 			
-		}catch( Throwable e ){
+		String	provider = null;
+		
+		for (int i=0;i<providers.length;i++){
+				
+			try{
+				Class.forName(providers[i]).newInstance();
+		
+				provider	 = providers[i];
+				
+				break;
+				
+			}catch( Throwable e ){
+			}
+		}
+		
+		if ( provider == null ){
 			
-			Debug.printStackTrace( e );
+			Debug.out( "No SSL provider available" );
 		}
 		
 		try{
@@ -97,10 +145,41 @@ SESecurityManagerImpl
 			
 		}catch( Throwable e ){
 			
-			LGLogger.log( LGLogger.ERROR, "Bouncy Castle not available" );
+			Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
+					"Bouncy Castle not available"));
 		}
 		
 		installSecurityManager();
+		
+		ensureStoreExists( keystore_name );
+		
+		ensureStoreExists( truststore_name );
+		
+		/*
+			try{
+				Certificate c = createSelfSignedCertificate( "Dummy", "CN=fred,OU=wap,O=wip,L=here,ST=there,C=GB", 512 );
+				
+				addCertToTrustStore( "SomeAlias", c);
+	
+				addCertToTrustStore( null, null );
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
+	
+		/*
+			try{
+				Certificate c = createSelfSignedCertificate( "SomeAlias", "CN=fred,OU=wap,O=wip,L=here,ST=there,C=GB", 1000 );
+			
+				addCertToTrustStore( "SomeAlias", c);
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
+		}
+		*/
 	}
 	
 	public String
@@ -183,7 +262,10 @@ SESecurityManagerImpl
 		try{
 			exit_vm_permitted	= true;
 			
-			System.exit( status );
+			try{
+      	System.exit( status );
+      }
+      catch( Throwable t ){}
 			
 		}finally{
 			
@@ -201,6 +283,7 @@ SESecurityManagerImpl
 					
 					protected PasswordAuthentication
 					getPasswordAuthentication()
+					
 					{			
 						try{
 							auth_mon.enter();
@@ -238,9 +321,19 @@ SESecurityManagerImpl
 		String		host,
 		int			port )
 	{
-		try{
-			this_mon.enter();
+			// special case for socks auth when user is explicitly "<none>" as some servers seem to cause
+			// a password prompt when no auth defined and java doesn't cache a successful blank response
+			// thus causing repetitive prompts
+		
+		if ( protocol.toLowerCase().startsWith( "socks" )){
 			
+			if ( COConfigurationManager.getStringParameter("Proxy.Username").trim().equalsIgnoreCase( "<none>" )){
+				
+				return( new PasswordAuthentication( "", "".toCharArray()));
+			}
+		}
+		
+		try{			
 			URL	tracker_url = new URL( protocol + "://" + host + ":" + port + "/" );
 		
 			return( getPasswordAuthentication( realm, tracker_url ));
@@ -250,10 +343,6 @@ SESecurityManagerImpl
 			Debug.printStackTrace( e );
 			
 			return( null );
-			
-		}finally{
-			
-			this_mon.exit();
 		}
 	}
 	
@@ -263,11 +352,9 @@ SESecurityManagerImpl
 		File	f  = new File(keystore_name);
 		
 		if ( !f.exists()){
-			
-			LGLogger.logUnrepeatableAlertUsingResource( 
-					LGLogger.AT_ERROR,
-					"Security.keystore.empty",
-					new String[]{ keystore_name });
+			Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
+					LogAlert.AT_ERROR, "Security.keystore.empty"),
+					new String[] { keystore_name });
 			
 			return( false );
 		}
@@ -278,21 +365,18 @@ SESecurityManagerImpl
 			Enumeration enumx = key_store.aliases();
 			
 			if ( !enumx.hasMoreElements()){
-				
-				LGLogger.logUnrepeatableAlertUsingResource( 
-						LGLogger.AT_ERROR,
-						"Security.keystore.empty",
-						new String[]{ keystore_name });
+				Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
+						LogAlert.AT_ERROR, "Security.keystore.empty"),
+						new String[] { keystore_name });
 				
 				return( false );			
 			}
 			
 		}catch( Throwable e ){
 		
-			LGLogger.logUnrepeatableAlertUsingResource( 
-					LGLogger.AT_ERROR,
-					"Security.keystore.corrupt",
-					new String[]{ keystore_name });
+			Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
+					LogAlert.AT_ERROR, "Security.keystore.corrupt"),
+					new String[] { keystore_name });
 			
 			return( false );			
 		}
@@ -300,6 +384,92 @@ SESecurityManagerImpl
 		return( true );
 	}
 	
+	protected boolean
+	ensureStoreExists(
+		String	name )
+	{
+		try{
+			this_mon.enter();
+		
+			KeyStore keystore = KeyStore.getInstance( KEYSTORE_TYPE );
+			
+			if ( !new File(name).exists()){
+		
+				keystore.load(null,null);
+			
+				FileOutputStream	out = null;
+				
+				try{
+					out = new FileOutputStream(name);
+			
+					keystore.store(out, SESecurityManager.SSL_PASSWORD.toCharArray());
+			
+				}finally{
+					
+					if ( out != null ){
+						
+						out.close();
+					}						
+				}
+				
+				return( true );
+				
+			}else{
+				
+				return( false );
+			}
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace(e);
+			
+			return( false );
+			
+		}finally{
+			
+			this_mon.exit();
+		}
+	}
+	
+	public KeyStore
+	getKeyStore()
+	
+		throws Exception
+	{
+		return( loadKeyStore());
+	}
+	
+	public KeyStore
+	getTrustStore()
+	
+		throws Exception
+	{
+		KeyStore keystore = KeyStore.getInstance( KEYSTORE_TYPE );
+		
+		if ( !new File(truststore_name).exists()){
+	
+			keystore.load(null,null);
+			
+		}else{
+		
+			FileInputStream		in 	= null;
+
+			try{
+				in = new FileInputStream(truststore_name);
+		
+				keystore.load(in, SESecurityManager.SSL_PASSWORD.toCharArray());
+				
+			}finally{
+				
+				if ( in != null ){
+					
+					in.close();
+				}
+			}
+		}
+		
+		return( keystore );
+	}
+	
 	protected KeyStore
 	loadKeyStore()
 	
@@ -316,7 +486,7 @@ SESecurityManagerImpl
 		
 		throws Exception
 	{
-		KeyStore key_store = KeyStore.getInstance("JKS");
+		KeyStore key_store = KeyStore.getInstance( KEYSTORE_TYPE );
 		
 		if ( !new File(keystore_name).exists()){
 			
@@ -422,7 +592,7 @@ SESecurityManagerImpl
 				});
 	}
 	
-	public void
+	public Certificate
 	createSelfSignedCertificate(
 		String		alias,
 		String		cert_dn,
@@ -430,10 +600,10 @@ SESecurityManagerImpl
 	
 		throws Exception
 	{
-		SESecurityManagerBC.createSelfSignedCertificate( this, alias, cert_dn, strength );
+		return( SESecurityManagerBC.createSelfSignedCertificate( this, alias, cert_dn, strength ));
 	}
 	
-	public boolean
+	public SSLSocketFactory
 	installServerCertificates(
 		URL		https_url )
 	{
@@ -481,7 +651,7 @@ SESecurityManagerImpl
 				
 				if ( serverCerts.length == 0 ){
 									
-					return( false );
+					return( null );
 				}
 				
 				java.security.cert.Certificate	cert = serverCerts[0];
@@ -514,19 +684,17 @@ SESecurityManagerImpl
 						
 						String	alias = host.concat(":").concat(String.valueOf(port));
 				
-						addCertToTrustStore( alias, cert );
-				
-						return( true );
+						return( addCertToTrustStore( alias, cert ));
 					}
 				}
 				
-				return( false );
+				return( null );
 				
 			}catch( Throwable e ){
 				
 				Debug.printStackTrace( e );
 				
-				return( false );
+				return( null );
 				
 			}finally{
 				
@@ -591,7 +759,7 @@ SESecurityManagerImpl
 		}
 	}
 	
-	protected void
+	protected SSLSocketFactory
 	addCertToTrustStore(
 		String							alias,
 		java.security.cert.Certificate	cert )
@@ -601,29 +769,7 @@ SESecurityManagerImpl
 		try{
 			this_mon.enter();
 		
-			KeyStore keystore = KeyStore.getInstance("JKS");
-			
-			if ( !new File(truststore_name).exists()){
-		
-				keystore.load(null,null);
-				
-			}else{
-			
-				FileInputStream		in 	= null;
-	
-				try{
-					in = new FileInputStream(truststore_name);
-			
-					keystore.load(in, SESecurityManager.SSL_PASSWORD.toCharArray());
-					
-				}finally{
-					
-					if ( in != null ){
-						
-						in.close();
-					}
-				}
-			}
+			KeyStore keystore = getTrustStore();
 			
 			if ( cert != null ){
 				
@@ -660,8 +806,11 @@ SESecurityManagerImpl
 			
 			ctx.init(null, tmf.getTrustManagers(), null);
 						
-			HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());
+			SSLSocketFactory	factory = ctx.getSocketFactory();
 			
+			HttpsURLConnection.setDefaultSSLSocketFactory( factory );
+			
+			return( factory );
 		}finally{
 			
 			this_mon.exit();
@@ -677,16 +826,29 @@ SESecurityManagerImpl
 		
 		if ( handler != null ){
 			
-			return(((SEPasswordListener)handler[0]).getAuthentication( realm, (URL)handler[1] ));
+			try{
+				return(((SEPasswordListener)handler[0]).getAuthentication( realm, (URL)handler[1] ));
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
 		}
 		
-		for (int i=0;i<password_listeners.size();i++){
+		Iterator	it = password_listeners.iterator();
+		
+		while( it.hasNext()){
 			
-			PasswordAuthentication res = ((SEPasswordListener)password_listeners.get(i)).getAuthentication( realm, tracker );
-			
-			if ( res != null ){
+			try{
+				PasswordAuthentication res = ((SEPasswordListener)it.next()).getAuthentication( realm, tracker );
 				
-				return( res );
+				if ( res != null ){
+					
+					return( res );
+				}
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
 			}
 		}
 		
@@ -699,9 +861,11 @@ SESecurityManagerImpl
 		URL			tracker,
 		boolean		success )
 	{
-		for (int i=0;i<password_listeners.size();i++){
+		Iterator	it = password_listeners.iterator();
+		
+		while( it.hasNext()){
 			
-			((SEPasswordListener)password_listeners.get(i)).setAuthenticationOutcome( realm, tracker, success );
+			((SEPasswordListener)it.next()).setAuthenticationOutcome( realm, tracker, success );
 		}
 	}
 		
@@ -735,6 +899,23 @@ SESecurityManagerImpl
 		}
 	}
 	
+	public void
+	clearPasswords()
+	{
+		Iterator	it = password_listeners.iterator();
+		
+		while( it.hasNext()){
+			
+			try{				
+				((SEPasswordListener)it.next()).clearPasswords();
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
+		}
+	}
+	
 	public void
 	addPasswordHandler(
 		URL						url,
