@@ -29,6 +29,7 @@ package org.gudy.azureus2.core3.tracker.server.impl;
 
 import java.util.*;
 import java.net.URL;
+import java.net.URLDecoder;
 
 import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.core3.config.*;
@@ -51,7 +52,19 @@ TRTrackerServerImpl
 	public static int		scrape_cache_period			= TRTrackerServer.DEFAULT_SCRAPE_CACHE_PERIOD;
 	public static int		announce_cache_threshold	= TRTrackerServer.DEFAULT_ANNOUNCE_CACHE_PEER_THRESHOLD;
 	public static int		max_seed_retention			= 0;
+	public static int		seed_limit					= 0;
+	public static boolean	full_scrape_enable			= true;
 	
+	public static boolean	all_networks_permitted		= true;
+	public static String[]	permitted_networks			= {};
+	
+		// torrent map is static across all protocol servers
+	
+	private static Map		torrent_map = new HashMap(); 
+	
+	protected AEMonitor class_mon 	= new AEMonitor( "TRTrackerServer:class" );
+
+
 	static{
 
 		COConfigurationManager.addListener(
@@ -72,7 +85,7 @@ TRTrackerServerImpl
 	{
 		send_peer_ids = COConfigurationManager.getBooleanParameter( "Tracker Send Peer IDs", true );
 				
-		max_peers_to_send = COConfigurationManager.getIntParameter( "Tracker Max Peers Returned", 0 );
+		max_peers_to_send = COConfigurationManager.getIntParameter( "Tracker Max Peers Returned" );
 				
 		scrape_cache_period = COConfigurationManager.getIntParameter( "Tracker Scrape Cache", TRTrackerServer.DEFAULT_SCRAPE_CACHE_PERIOD );
 				
@@ -81,6 +94,35 @@ TRTrackerServerImpl
 		announce_cache_threshold = COConfigurationManager.getIntParameter( "Tracker Announce Cache Min Peers", TRTrackerServer.DEFAULT_ANNOUNCE_CACHE_PEER_THRESHOLD );
 
 		max_seed_retention = COConfigurationManager.getIntParameter( "Tracker Max Seeds Retained", 0 );
+		
+		seed_limit = COConfigurationManager.getIntParameter( "Tracker Max Seeds", 0 );
+		
+		List	nets = new ArrayList();
+		
+		for (int i=0;i<AENetworkClassifier.AT_NETWORKS.length;i++){
+			
+			String	net = AENetworkClassifier.AT_NETWORKS[i];
+			
+			boolean	enabled = 
+				COConfigurationManager.getBooleanParameter(
+						"Tracker Network Selection Default." + net );
+			
+			if ( enabled ){
+				
+				nets.add( net );
+			}
+		}
+		
+		String[]	s_nets = new String[nets.size()];
+		
+		nets.toArray(s_nets);
+		
+		permitted_networks	= s_nets;
+		
+		all_networks_permitted = s_nets.length == AENetworkClassifier.AT_NETWORKS.length;
+		
+		full_scrape_enable = COConfigurationManager.getBooleanParameter( "Tracker Server Full Scrape Enable" );
+
 	}
 	
 	protected static boolean
@@ -119,34 +161,60 @@ TRTrackerServerImpl
 		return( max_seed_retention );
 	}
 	
+	protected static int
+	getSeedLimit()
+	{
+		return( seed_limit );
+	}
+	
+	public static boolean
+	isFullScrapeEnabled()
+	{
+		return( full_scrape_enable );
+	}
+	
+	protected static boolean
+	getAllNetworksSupported()
+	{
+		return( all_networks_permitted );
+	}
+	
+	protected static String[]
+	getPermittedNetworks()
+	{
+		return( permitted_networks );
+	}
+	
 	protected IpFilter	ip_filter	= IpFilterManagerFactory.getSingleton().getIPFilter();
 	
-	protected Map		torrent_map = new HashMap(); 
+	private long		current_announce_retry_interval;
+	private long		current_scrape_retry_interval;
+	private long		current_total_clients;
 	
-	protected long		current_announce_retry_interval;
-	protected long		current_scrape_retry_interval;
-	protected long		current_total_clients;
+	private int		current_min_poll_interval;
 	
-	protected int		current_min_poll_interval;
-	
-	protected TRTrackerServerStatsImpl	stats = new TRTrackerServerStatsImpl();
+	private TRTrackerServerStatsImpl	stats = new TRTrackerServerStatsImpl();
 		
-	protected String	name;
-	protected boolean	web_password_enabled;
-	protected boolean	web_password_https_only;
+	private String	name;
+	private boolean	web_password_enabled;
+	private boolean	web_password_https_only;
 	
-	protected boolean	tracker_password_enabled;
-	protected String	password_user;
-	protected byte[]	password_pw;
-	protected boolean	compact_enabled;
-	protected boolean	key_enabled;
+	private boolean	tracker_password_enabled;
+	private String	password_user;
+	private byte[]	password_pw;
+	private boolean	compact_enabled;
+	private boolean	key_enabled;
 	
 
 	protected Vector	listeners 			= new Vector();
-	protected List		auth_listeners		= new ArrayList();
+	private List		auth_listeners		= new ArrayList();
+	
+	private Vector	request_listeners 	= new Vector();
 	
 	protected AEMonitor this_mon 	= new AEMonitor( "TRTrackerServer" );
 
+	private COConfigurationListener		config_listener;
+	private boolean						destroyed;
 	
 	public
 	TRTrackerServerImpl(
@@ -154,23 +222,31 @@ TRTrackerServerImpl
 	{
 		name		= _name==null?DEFAULT_NAME:_name;
 
-		COConfigurationManager.addListener(
-				new COConfigurationListener()
+		config_listener = 
+			new COConfigurationListener()
+			{
+				public void
+				configurationSaved()
 				{
-					public void
-					configurationSaved()
-					{
-						readConfigSettings();
-					}
-				});
-				
+					readConfigSettings();
+				}
+			};
+		
+		COConfigurationManager.addListener( config_listener );
+
 		readConfigSettings();
 					
 		current_min_poll_interval	= COConfigurationManager.getIntParameter("Tracker Poll Interval Min", DEFAULT_MIN_RETRY_DELAY );
-		int	scrape_percentage 		= COConfigurationManager.getIntParameter("Tracker Scrape Retry Percentage", DEFAULT_SCRAPE_RETRY_PERCENTAGE );
 		
-		current_announce_retry_interval = current_min_poll_interval;
+		if ( current_min_poll_interval < RETRY_MINIMUM_SECS ){
+			
+			current_min_poll_interval = RETRY_MINIMUM_SECS;
+		}
 		
+		current_announce_retry_interval = current_min_poll_interval;		
+
+		int	scrape_percentage 		= COConfigurationManager.getIntParameter("Tracker Scrape Retry Percentage", DEFAULT_SCRAPE_RETRY_PERCENTAGE );		
+	
 		current_scrape_retry_interval	= (current_announce_retry_interval*scrape_percentage)/100;
 		
 		Thread timer_thread = 
@@ -314,6 +390,12 @@ TRTrackerServerImpl
 		return( password_pw );
 	}
 	
+	public long
+	getMinAnnounceRetryInterval()
+	{
+		return( current_min_poll_interval );
+	}
+	
 	public long
 	getAnnounceRetryInterval(
 		TRTrackerServerTorrentImpl	torrent )
@@ -356,6 +438,12 @@ TRTrackerServerImpl
 		return( res );
 	}
 	
+	public long
+	getMinScrapeRetryInterval()
+	{
+		return( current_min_poll_interval );
+	}
+	
 	public TRTrackerServerStats
 	getStats()
 	{
@@ -369,7 +457,7 @@ TRTrackerServerImpl
 		int							bytes_out )
 	{
 		try{
-			this_mon.enter();
+			class_mon.enter();
 		
 			stats.update( bytes_in, bytes_out );
 			
@@ -411,7 +499,7 @@ TRTrackerServerImpl
 			}
 		}finally{
 			
-			this_mon.exit();
+			class_mon.exit();
 		}
 	}
 	
@@ -420,7 +508,7 @@ TRTrackerServerImpl
 	{
 		long	time_to_go = TIMEOUT_CHECK;
 		
-		while(true){
+		while( !destroyed ){
 			
 			try{
 				Thread.sleep( RETRY_MINIMUM_MILLIS );
@@ -431,6 +519,11 @@ TRTrackerServerImpl
 				
 				current_min_poll_interval 	= COConfigurationManager.getIntParameter("Tracker Poll Interval Min", DEFAULT_MIN_RETRY_DELAY );
 				
+				if ( current_min_poll_interval < RETRY_MINIMUM_SECS ){
+					
+					current_min_poll_interval = RETRY_MINIMUM_SECS;
+				}
+				
 				int	min		= current_min_poll_interval;
 				int	max 	= COConfigurationManager.getIntParameter("Tracker Poll Interval Max", DEFAULT_MAX_RETRY_DELAY );
 				int	inc_by 	= COConfigurationManager.getIntParameter("Tracker Poll Inc By", DEFAULT_INC_BY );
@@ -443,7 +536,7 @@ TRTrackerServerImpl
 				int	clients = 0;
 				
 				try{
-					this_mon.enter();
+					class_mon.enter();
 					
 					Iterator	it = torrent_map.values().iterator();
 					
@@ -455,7 +548,7 @@ TRTrackerServerImpl
 					}
 				}finally{
 					
-					this_mon.exit();
+					class_mon.exit();
 				}
 				
 				if ( inc_by > 0 && inc_per > 0 ){
@@ -486,7 +579,7 @@ TRTrackerServerImpl
 					time_to_go = TIMEOUT_CHECK;
 					
 					try{
-						this_mon.enter();
+						class_mon.enter();
 						
 						Iterator	it = torrent_map.values().iterator();
 						
@@ -498,7 +591,7 @@ TRTrackerServerImpl
 						}
 					}finally{
 						
-						this_mon.exit();
+						class_mon.exit();
 					}
 				}
 				
@@ -515,6 +608,17 @@ TRTrackerServerImpl
 		byte[]		_hash,
 		boolean		_explicit )
 	
+		throws TRTrackerServerException
+	{
+		return( permit( _hash, _explicit, true ));
+	}
+	
+	public TRTrackerServerTorrent
+	permit(
+		byte[]		_hash,
+		boolean		_explicit,
+		boolean		_enabled )
+	
 		throws TRTrackerServerException
 	{
 		// System.out.println( "TRTrackerServerImpl::permit( " + _explicit + ")");
@@ -523,7 +627,17 @@ TRTrackerServerImpl
 		
 			// don't invoke listeners when synched, deadlock possible
 		
-		TRTrackerServerTorrentImpl	entry = (TRTrackerServerTorrentImpl)torrent_map.get( hash );
+		TRTrackerServerTorrentImpl	entry;
+		
+		try{
+			class_mon.enter();
+			
+			entry = (TRTrackerServerTorrentImpl)torrent_map.get( hash );
+			
+		}finally{
+			
+			class_mon.exit();
+		}
 		
 		if ( entry == null ){
 			
@@ -534,22 +648,24 @@ TRTrackerServerImpl
 					throw( new TRTrackerServerException( "operation denied"));			
 				}
 			}
-		}
 		
-		try{
-			this_mon.enter();
-		
-			entry = (TRTrackerServerTorrentImpl)torrent_map.get( hash );
+			try{
+				class_mon.enter();
 			
-			if ( entry == null ){
-			
-				entry = new TRTrackerServerTorrentImpl( hash );
-			
-				torrent_map.put( hash, entry );
+					// double check in-case added in parallel
+				
+				entry = (TRTrackerServerTorrentImpl)torrent_map.get( hash );
+				
+				if ( entry == null ){
+				
+					entry = new TRTrackerServerTorrentImpl( hash, _enabled );
+				
+					torrent_map.put( hash, entry );
+				}
+			}finally{
+				
+				class_mon.exit();
 			}
-		}finally{
-			
-			this_mon.exit();
 		}
 		
 		return( entry );
@@ -575,7 +691,7 @@ TRTrackerServerImpl
 		}
 
 		try{
-			this_mon.enter();
+			class_mon.enter();
 			
 			TRTrackerServerTorrentImpl	entry = (TRTrackerServerTorrentImpl)torrent_map.get( hash );
 	
@@ -588,7 +704,7 @@ TRTrackerServerImpl
 			
 		}finally{
 			
-			this_mon.exit();
+			class_mon.exit();
 		}
 	}
 	
@@ -596,14 +712,22 @@ TRTrackerServerImpl
 	getTorrent(
 		byte[]		hash )
 	{
-		return((TRTrackerServerTorrentImpl)torrent_map.get(new HashWrapper(hash)));
+		try{
+			class_mon.enter();
+		
+			return((TRTrackerServerTorrentImpl)torrent_map.get(new HashWrapper(hash)));
+			
+		}finally{
+			
+			class_mon.exit();
+		}
 	}
 	
 	public TRTrackerServerTorrentImpl[]
 	getTorrents()
 	{
 		try{
-			this_mon.enter();
+			class_mon.enter();
 		
 			TRTrackerServerTorrentImpl[]	res = new TRTrackerServerTorrentImpl[torrent_map.size()];
 			
@@ -612,7 +736,7 @@ TRTrackerServerImpl
 			return( res );	
 		}finally{
 			
-			this_mon.exit();
+			class_mon.exit();
 		}
 	}
 	
@@ -688,4 +812,148 @@ TRTrackerServerImpl
 	{
 		auth_listeners.remove(l);
 	}
+	
+	public void
+	preProcess(
+		TRTrackerServerPeer			peer,
+		TRTrackerServerTorrent		torrent,
+		int							type,
+		String						request,
+		Map							response )
+	
+		throws TRTrackerServerException
+	{
+		if ( request_listeners.size() > 0 ){
+			
+				// if this is a scrape then we need to patch up stuff as it may be multi-scrape
+			
+			if ( type == TRTrackerServerRequest.RT_SCRAPE ){
+				
+				try{
+					int	request_pos = 10;
+					
+					while( true ){
+						
+						int	p = request.indexOf( "info_hash=", request_pos );
+						
+						String	bit;
+						
+						if ( p == -1 ){
+						
+							if ( request_pos == 10 ){
+								
+								break;	// only one entry, nothing to do
+							}
+							
+							bit = request.substring( request_pos );
+							
+						}else{
+							
+							bit = request.substring( request_pos, p );
+						}
+														
+						int	pos = bit.indexOf('&');
+						
+						String	hash_str = pos==-1?bit:bit.substring(0,pos);
+						
+						hash_str = URLDecoder.decode( hash_str, Constants.BYTE_ENCODING );
+						
+						byte[]	hash = hash_str.getBytes(Constants.BYTE_ENCODING);
+						
+						if ( Arrays.equals( hash, torrent.getHash().getBytes())){
+							
+							request = "info_hash=" + bit;
+							
+							if ( request.endsWith("&")){
+								
+								request = request.substring(0,request.length()-1);
+							}
+							
+							break;
+						}
+						
+						if ( p == -1 ){
+							
+							break;
+						}
+						
+						request_pos = p + 10;
+					}
+				}catch( Throwable e ){
+						
+					Debug.printStackTrace(e);
+				}
+			}
+			
+			TRTrackerServerRequestImpl	req = new TRTrackerServerRequestImpl( this, peer, torrent, type, request, response );
+			
+			for (int i=0;i<request_listeners.size();i++){
+				
+				try{
+					((TRTrackerServerRequestListener)request_listeners.elementAt(i)).preProcess( req );
+					
+				}catch( TRTrackerServerException e ){
+				
+					throw( e );
+					
+				}catch( Throwable e ){
+					
+					Debug.printStackTrace( e );
+				}
+			}
+		}
+	}
+	
+	public void
+	postProcess(
+		TRTrackerServerPeer			peer,
+		TRTrackerServerTorrentImpl	torrent,
+		int							type,
+		String						request,
+		Map							response )
+	
+		throws TRTrackerServerException
+	{
+		if ( request_listeners.size() > 0 ){
+			
+			TRTrackerServerRequestImpl	req = new TRTrackerServerRequestImpl( this, peer, torrent, type, request, response );
+			
+			for (int i=0;i<request_listeners.size();i++){
+				
+				try{
+					((TRTrackerServerRequestListener)request_listeners.elementAt(i)).postProcess( req );
+				
+				}catch( TRTrackerServerException e ){
+					
+					throw( e );
+					
+				}catch( Throwable e ){
+					
+					Debug.printStackTrace( e );
+				}
+			}
+		}
+	}
+		
+	public void
+	addRequestListener(
+		TRTrackerServerRequestListener	l )
+	{
+		request_listeners.addElement( l );
+	}
+	
+	public void
+	removeRequestListener(
+		TRTrackerServerRequestListener	l )
+	{
+		request_listeners.removeElement(l);
+	}
+	
+	protected void
+	destroy()
+	{
+		destroyed	= true;
+		
+		COConfigurationManager.removeListener( config_listener );
+	}
 }
