@@ -23,6 +23,7 @@ package org.gudy.azureus2.plugins.peers;
 
 import org.gudy.azureus2.plugins.download.*;
 import org.gudy.azureus2.plugins.disk.*;
+import org.gudy.azureus2.plugins.utils.PooledByteBuffer;
 
 /**
  * @author parg
@@ -39,11 +40,42 @@ PeerManager
 	public void
 	addPeer(
 		Peer		peer );
+  
+  
+	  /**
+	   * Add a new peer, using the default internal Peer implementation
+	   * (like for peers given in announce reply), using the given address
+	   * and port.
+	   * @param ip_address of peer to inject
+	   * @param port of peer to inject
+	   * @deprecated
+	   */
 	
+	public void 
+	addPeer( 
+		String ip_address, 
+		int port );
+	
+	public void 
+	addPeer( 
+		String 	ip_address, 
+		int 	tcp_port, 
+		boolean use_crypto );
+  
+	public void 
+	addPeer( 
+		String 	ip_address, 
+		int 	tcp_port, 
+		int		udp_port,
+		boolean use_crypto );
+  
 	public void
 	removePeer(
 		Peer		peer );
 	
+	public Peer[]
+	getPeers();
+	
 	public DiskManager
 	getDiskManager();
 	
@@ -53,8 +85,26 @@ PeerManager
 	public boolean
 	isSeeding();
 	
+	public boolean
+	isSuperSeeding();
+	
 	public PeerStats
-	createPeerStats();
+	createPeerStats(
+		Peer		peer );
+	
+	public void 
+	requestComplete(
+		PeerReadRequest		request,
+		PooledByteBuffer 	data,
+		Peer 				sender);
+	
+	public void
+	requestCancelled(
+		PeerReadRequest		request,
+		Peer				sender );
+	
+	public Piece[]
+	getPieces();
 	
 	public void
 	addListener(
