@@ -21,6 +21,10 @@
 
 package org.gudy.azureus2.plugins.utils;
 
+import java.io.IOException;
+import java.util.Comparator;
+import java.util.Map;
+
 /**
  * @author parg
  *
@@ -60,4 +64,39 @@ Formatters
 	public String
 	formatTimeFromSeconds(
 		long		seconds );
+	
+	
+	/**
+	 * Format seconds remaining into an ETA value.
+	 * @param seconds
+	 * @return
+	 * @since 2.4.0.3
+	 */
+	public String formatETAFromSeconds( long seconds );
+	
+	
+	
+	public byte[]
+	bEncode(
+		Map	map )
+	
+		throws IOException;
+	
+	public Map
+	bDecode(
+		byte[]	data )
+	
+		throws IOException;
+	
+	public String
+	base32Encode(
+		byte[]		data );
+	
+	public byte[]
+	base32Decode(
+		String		data );
+	
+	public Comparator
+	getAlphanumericComparator(
+		boolean	ignore_case );
 }
