@@ -31,6 +31,19 @@ Semaphore
 	public void
 	reserve();
 	
+	public boolean
+	reserveIfAvailable();
+	
+		/**
+		 * reserve a semaphore subject to timeout 
+		 * @param timeout_millis	time to wait
+		 * @return true -> semaphore reserved, false -> timeout exceeded
+		 */
+	
+	public boolean
+	reserve(
+		long	timeout_millis );
+	
 	public void
 	release();
 }
