@@ -26,6 +26,7 @@ package org.gudy.azureus2.pluginsimpl.local.download;
  *
  */
 
+import java.io.File;
 import java.util.*;
 
 import org.gudy.azureus2.core3.util.*;
@@ -40,10 +41,15 @@ import org.gudy.azureus2.plugins.torrent.Torrent;
 import org.gudy.azureus2.plugins.torrent.TorrentAttribute;
 
 import org.gudy.azureus2.plugins.peers.*;
+import org.gudy.azureus2.pluginsimpl.local.disk.DiskManagerFileInfoImpl;
 import org.gudy.azureus2.pluginsimpl.local.peers.*;
 import org.gudy.azureus2.pluginsimpl.local.torrent.TorrentImpl;
 import org.gudy.azureus2.pluginsimpl.local.torrent.TorrentManagerImpl;
+import org.gudy.azureus2.plugins.disk.DiskManager;
+import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.plugins.download.Download;
+import org.gudy.azureus2.plugins.download.DownloadActivationEvent;
+import org.gudy.azureus2.plugins.download.DownloadActivationListener;
 import org.gudy.azureus2.plugins.download.DownloadListener;
 import org.gudy.azureus2.plugins.download.DownloadPeerListener;
 import org.gudy.azureus2.plugins.download.DownloadPropertyListener;
@@ -55,33 +61,42 @@ import org.gudy.azureus2.plugins.download.DownloadStats;
 import org.gudy.azureus2.plugins.download.DownloadException;
 import org.gudy.azureus2.plugins.download.DownloadRemovalVetoException;
 import org.gudy.azureus2.plugins.download.DownloadWillBeRemovedListener;
+import org.gudy.azureus2.plugins.download.session.SessionAuthenticator;
 
 import org.gudy.azureus2.core3.internat.MessageText;
+import org.gudy.azureus2.core3.logging.LogRelation;
+
+import com.aelitis.azureus.core.util.CopyOnWriteList;
 
 public class 
 DownloadImpl
+	extends LogRelation
 	implements 	Download, DownloadManagerListener, 
 				DownloadManagerTrackerListener, DownloadManagerPeerListener,
-				DownloadManagerStateListener
+				DownloadManagerStateListener, DownloadManagerActivationListener
 {
-	protected DownloadManager		download_manager;
-	protected DownloadStatsImpl		download_stats;
+	private DownloadManager		download_manager;
+	private DownloadStatsImpl		download_stats;
 	
-	protected int		latest_state		= ST_STOPPED;
-	protected boolean 	latest_forcedStart;
+	private int		latest_state		= ST_STOPPED;
+	private boolean 	latest_forcedStart;
 	
-	protected DownloadAnnounceResultImpl	last_announce_result 	= new DownloadAnnounceResultImpl(this,null);
-	protected DownloadScrapeResultImpl		last_scrape_result		= new DownloadScrapeResultImpl( this, null );
+	private DownloadAnnounceResultImpl	last_announce_result 	= new DownloadAnnounceResultImpl(this,null);
+	private DownloadScrapeResultImpl		last_scrape_result		= new DownloadScrapeResultImpl( this, null );
+    private TorrentImpl torrent = null;
 	
-	protected List		listeners 				= new ArrayList();
-	protected AEMonitor	listeners_mon			= new AEMonitor( "Download:L");
-	protected List		property_listeners		= new ArrayList();
-	protected List		tracker_listeners		= new ArrayList();
-	protected AEMonitor	tracker_listeners_mon	= new AEMonitor( "Download:TL");
-	protected List		removal_listeners 		= new ArrayList();
-	protected AEMonitor	removal_listeners_mon	= new AEMonitor( "Download:RL");
-	protected List		peer_listeners			= new ArrayList();
-	protected AEMonitor	peer_listeners_mon		= new AEMonitor( "Download:PL");
+	private List		listeners 				= new ArrayList();
+	private AEMonitor	listeners_mon			= new AEMonitor( "Download:L");
+	private List		property_listeners		= new ArrayList();
+	private List		tracker_listeners		= new ArrayList();
+	private AEMonitor	tracker_listeners_mon	= new AEMonitor( "Download:TL");
+	private List		removal_listeners 		= new ArrayList();
+	private AEMonitor	removal_listeners_mon	= new AEMonitor( "Download:RL");
+	private List		peer_listeners			= new ArrayList();
+	private AEMonitor	peer_listeners_mon		= new AEMonitor( "Download:PL");
+	
+	private CopyOnWriteList	activation_listeners	= new CopyOnWriteList();
+	private DownloadActivationEvent	activation_state;
 	
 	protected
 	DownloadImpl(
@@ -90,24 +105,64 @@ DownloadImpl
 		download_manager	= _dm;
 		download_stats		= new DownloadStatsImpl( download_manager );
 		
+		activation_state = 
+			new DownloadActivationEvent()
+			{
+				public Download
+				getDownload()
+				{
+					return( DownloadImpl.this );
+				}
+				
+				public int
+				getActivationCount()
+				{
+					return( download_manager.getActivationCount());
+				}
+			};
+			
 		download_manager.addListener( this );
 		
 		latest_forcedStart = download_manager.isForceStart();
 	}
 	
-	protected DownloadManager
+	// Not available to plugins
+	public DownloadManager
 	getDownload()
 	{
 		return( download_manager );
 	}
-	
+
 	public int
 	getState()
 	{
-			// latest_state is maintained by the stateChange listener from
-			// the download manager
+		return( convertState( download_manager.getState()) );
+	}
+	
+	public int
+	getSubState()
+	{
+		int	state = getState();
 		
-		return( latest_state );
+		if ( state == ST_STOPPING ){
+			
+			int	substate = download_manager.getSubState();
+			
+			if ( substate == DownloadManager.STATE_QUEUED ){
+				
+				return( ST_QUEUED );
+				
+			}else if ( substate == DownloadManager.STATE_STOPPED ){
+				
+				return( ST_STOPPED );
+				
+			}else if ( substate == DownloadManager.STATE_ERROR ){
+				
+				return( ST_ERROR );
+			}
+		}
+		
+		return( state );
 	}
 	
 	protected int
@@ -199,39 +254,45 @@ DownloadImpl
 		return( download_manager.getErrorDetails());
 	}
 	
+	public boolean
+	getFlag(
+		long		flag )
+	{
+		return( download_manager.getDownloadState().getFlag( flag ));
+	}
+	
 	public int
 	getIndex()
 	{
-		return( download_manager.getIndex());
+		GlobalManager globalManager = download_manager.getGlobalManager();
+		return globalManager.getIndexOf(download_manager);
 	}
 	
-	public Torrent
-	getTorrent()
-	{
-		TOTorrent	torrent = download_manager.getTorrent();
-		
-		if ( torrent == null ){
-			
-			return( null );
-			
-		}else{
-			
-			return( new TorrentImpl( torrent ));
-		}
-	}
-
+    public Torrent
+    getTorrent()
+    {
+    	if (this.torrent != null) {return this.torrent;}
+    	
+        TOTorrent torrent = download_manager.getTorrent();
+        if (torrent == null) {return null;}
+        this.torrent = new TorrentImpl(torrent);
+        return this.torrent;
+    }
+    
 	public void
 	initialize()
 	
 		throws DownloadException
 	{
-		if ( download_manager.getState() == DownloadManager.STATE_WAITING ){
+		int	state = download_manager.getState();
+		
+		if ( state == DownloadManager.STATE_WAITING ){
 			
 			download_manager.initialize();
 			
 		}else{
 			
-			throw( new DownloadException( "Download::initialize: download not waiting" ));
+			throw( new DownloadException( "Download::initialize: download not waiting (state=" + state + ")" ));
 		}
 	}
 	
@@ -240,18 +301,15 @@ DownloadImpl
 	
 		throws DownloadException
 	{
-		if ( download_manager.getState() == DownloadManager.STATE_READY ){
+		int	state = download_manager.getState();
+		
+		if ( state == DownloadManager.STATE_READY ){
 						
 			download_manager.startDownload();
-							
-				//set previous hash fails and discarded values
-				
-			download_manager.getStats().setSavedDiscarded();
-			download_manager.getStats().setSavedHashFails();
-			
+										
 		}else{
 			
-			throw( new DownloadException( "Download::start: download not ready" ));
+			throw( new DownloadException( "Download::start: download not ready (state=" + state + ")" ));
 		}
 	}
 	
@@ -260,14 +318,16 @@ DownloadImpl
 	
 		throws DownloadException
 	{
-		if ( download_manager.getState() == DownloadManager.STATE_STOPPED ||
-		     download_manager.getState() == DownloadManager.STATE_QUEUED ){
+		int	state = download_manager.getState();
+		
+		if ( 	state == DownloadManager.STATE_STOPPED ||
+				state == DownloadManager.STATE_QUEUED ){
 			
-			download_manager.setState( DownloadManager.STATE_WAITING );
+			download_manager.setStateWaiting();
 			
 		}else{
 			
-			throw( new DownloadException( "Download::restart: download already running" ));
+			throw( new DownloadException( "Download::restart: download already running (state=" + state + ")" ));
 		}
 	}
 	
@@ -291,10 +351,27 @@ DownloadImpl
 	
 		throws DownloadException
 	{
-		if ( download_manager.getState() != DownloadManager.STATE_QUEUED)
+		if ( download_manager.getState() != DownloadManager.STATE_QUEUED){
+						
 			download_manager.stopIt( DownloadManager.STATE_QUEUED, false, false );
-		else
+			
+		}else{
+			
 			throw( new DownloadException( "Download::stopAndQueue: download already queued" ));
+		}
+	}
+	
+	public void
+	recheckData()
+	
+		throws DownloadException
+	{
+		if ( !download_manager.canForceRecheck()){
+			
+			throw( new DownloadException( "Download::recheckData: download must be stopped, quued or in error state" ));
+		}
+		
+		download_manager.forceRecheck();
 	}
 	
 	public boolean
@@ -315,6 +392,12 @@ DownloadImpl
 		download_manager.setForceStart(forceStart);
 	}
 	
+	public boolean
+	isPaused()
+	{
+		return( download_manager.isPaused());
+	}
+	
 	public int
 	getPosition()
 	{
@@ -336,13 +419,20 @@ DownloadImpl
 	public void
 	moveUp()
 	{
-		download_manager.moveUp();
+		download_manager.getGlobalManager().moveUp(download_manager);
 	}
 	
 	public void
 	moveDown()
 	{
-		download_manager.moveDown();
+		download_manager.getGlobalManager().moveDown(download_manager);
+	}
+	
+	public void
+	moveTo(
+		int	pos )
+	{
+		download_manager.getGlobalManager().moveTo( download_manager, pos );
 	}
 	
 	public String 
@@ -380,6 +470,47 @@ DownloadImpl
   	return( null );
   }
   
+  public String[]
+  getListAttribute(
+  	TorrentAttribute		attribute )
+  {
+	  	String	name = convertAttribute( attribute );
+	  	
+	  	if ( name != null ){
+	  		
+	  		return( download_manager.getDownloadState().getListAttribute( name ));
+	  	}
+	  	
+	  	return( null );
+  }
+  
+  public void
+  setMapAttribute(
+	TorrentAttribute		attribute,
+	Map						value )
+  {
+	  	String	name = convertAttribute( attribute );
+	  	
+	  	if ( name != null ){
+			
+			download_manager.getDownloadState().setMapAttribute( name, value );
+	  	}
+  }
+  
+  public Map
+  getMapAttribute(
+	TorrentAttribute		attribute )
+  {
+	  	String	name = convertAttribute( attribute );
+	  	
+	  	if ( name != null ){
+	  		
+	  		return( download_manager.getDownloadState().getMapAttribute( name ));
+	  	}
+	  	
+	  	return( null );
+  }
+  
   public void
   setAttribute(
   	TorrentAttribute		attribute,
@@ -397,10 +528,40 @@ DownloadImpl
   convertAttribute(
   	TorrentAttribute		attribute )
   {
-  	if ( attribute.getName() == TorrentAttribute.TA_CATEGORY ){
+ 	if ( attribute.getName() == TorrentAttribute.TA_CATEGORY ){
   		
   		return( DownloadManagerState.AT_CATEGORY );
   		
+ 	}else if ( attribute.getName() == TorrentAttribute.TA_NETWORKS ){
+  		
+		return( DownloadManagerState.AT_NETWORKS );
+		
+ 	}else if ( attribute.getName() == TorrentAttribute.TA_TRACKER_CLIENT_EXTENSIONS ){
+  		
+		return( DownloadManagerState.AT_TRACKER_CLIENT_EXTENSIONS );
+	  		
+	}else if ( attribute.getName() == TorrentAttribute.TA_PEER_SOURCES ){
+  		
+		return( DownloadManagerState.AT_PEER_SOURCES );
+
+	}else if ( attribute.getName() == TorrentAttribute.TA_DISPLAY_NAME ){
+		
+		return( DownloadManagerState.AT_DISPLAY_NAME );
+		
+	}else if ( attribute.getName() == TorrentAttribute.TA_CONTENT_MAP ){
+		
+		return( DownloadManagerState.AT_CONTENT_MAP );
+		
+	}else if ( attribute.getName() == TorrentAttribute.TA_SHARE_PROPERTIES ){
+  		
+			// this is a share-level attribute only, not propagated to individual downloads
+		
+		return( null );
+  		
+	}else if ( attribute.getName().startsWith( "Plugin." )){
+  		
+		return( attribute.getName());
+  		
   	}else{
   		
   		Debug.out( "Can't convert attribute '" + attribute.getName() + "'" );
@@ -413,10 +574,34 @@ DownloadImpl
   convertAttribute(
   	String			name )
   {
-  	if ( name.equals( DownloadManagerState.AT_CATEGORY )){
+ 	if ( name.equals( DownloadManagerState.AT_CATEGORY )){
   		
   		return( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_CATEGORY ));
   		
+	}else if ( name.equals( DownloadManagerState.AT_NETWORKS )){
+	  		
+	  	return( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_NETWORKS ));
+	  		
+	}else if ( name.equals( DownloadManagerState.AT_PEER_SOURCES )){
+  		
+		return( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_PEER_SOURCES ));
+		
+	}else if ( name.equals( DownloadManagerState.AT_TRACKER_CLIENT_EXTENSIONS )){
+  		
+		return( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_TRACKER_CLIENT_EXTENSIONS ));
+	
+	}else if ( name.equals ( DownloadManagerState.AT_DISPLAY_NAME)){
+		
+		return ( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_DISPLAY_NAME ));
+		
+	}else if ( name.equals ( DownloadManagerState.AT_CONTENT_MAP)){
+		
+		return ( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_CONTENT_MAP ));
+
+	}else if ( name.startsWith( "Plugin." )){
+  		
+		return( TorrentManagerImpl.getSingleton().getAttribute( name ));
+	  		
   	}else{
   		
   		return( null );
@@ -437,6 +622,16 @@ DownloadImpl
 	public void
 	remove()
 	
+		throws DownloadException, DownloadRemovalVetoException
+	{
+		remove( false, false );
+	}
+	
+	public void
+	remove(
+		boolean	delete_torrent,
+		boolean	delete_data )
+	
 		throws DownloadException, DownloadRemovalVetoException
 	{
 		int	dl_state = download_manager.getState();
@@ -448,6 +643,13 @@ DownloadImpl
 			GlobalManager globalManager = download_manager.getGlobalManager();
 			
 			try{
+				globalManager.canDownloadManagerBeRemoved(download_manager );
+				
+				if ( delete_torrent || delete_data ){
+					
+					download_manager.stopIt( dl_state, delete_torrent, delete_data );
+				}
+				
 				globalManager.removeDownloadManager(download_manager);
 				
 			}catch( GlobalManagerDownloadRemovalVetoException e ){
@@ -479,7 +681,7 @@ DownloadImpl
 				
 			}catch( GlobalManagerDownloadRemovalVetoException e ){
 				
-				throw( new DownloadRemovalVetoException( e.getMessage()));
+				throw( new DownloadRemovalVetoException( e.getMessage(),e.isSilent()));
 			}
 			
 		}else{
@@ -501,7 +703,24 @@ DownloadImpl
  	{
  		int	state = getState();
  		
- 		return( state == ST_SEEDING || download_manager.isDownloadComplete());
+ 		return state == ST_SEEDING || download_manager.isDownloadComplete(false);
+ 	}
+
+ 	public boolean isComplete(boolean bIncludeDND) {
+ 		return download_manager.isDownloadComplete(bIncludeDND);
+ 	}
+
+ 	public boolean
+ 	isChecking()
+ 	{
+ 		org.gudy.azureus2.core3.disk.DiskManager	dm = download_manager.getDiskManager();
+ 		
+ 		if ( dm != null ){
+ 			
+ 			return( dm.getCompleteRecheckStatus() != -1 );
+ 		}
+ 		
+ 		return( false );
  	}
  	
 	protected void
@@ -576,7 +795,12 @@ DownloadImpl
 		boolean 			bCompleted) 
 	{
 	}
-	
+
+	public void
+	filePriorityChanged( DownloadManager download, org.gudy.azureus2.core3.disk.DiskManagerFileInfo file )
+	{	  
+	}
+	  
   public void 
   positionChanged(
   	DownloadManager download, 
@@ -631,6 +855,13 @@ DownloadImpl
 	public DownloadAnnounceResult
 	getLastAnnounceResult()
 	{
+		TRTrackerAnnouncer tc = download_manager.getTrackerClient();
+		
+		if ( tc != null ){
+			
+			last_announce_result.setContent( tc.getLastResponse());
+		}
+		
 		return( last_announce_result );
 	}
 	
@@ -665,7 +896,7 @@ DownloadImpl
 	
 	public void
 	announceResult(
-		TRTrackerResponse			response )
+		TRTrackerAnnouncerResponse			response )
 	{
 		last_announce_result.setContent( response );
 		
@@ -683,50 +914,66 @@ DownloadImpl
 		}
 	}
 	
+	public void
+	setAnnounceResult(
+		DownloadAnnounceResult	result )
+	{
+		download_manager.setAnnounceResult( result );
+	}
+	
+	public void
+	setScrapeResult(
+		DownloadScrapeResult	result )
+	{
+		download_manager.setScrapeResult( result );
+	}
+	
 	public void
 	stateChanged(
 		DownloadManagerState			state,
 		DownloadManagerStateEvent		event )
 	{
-		if ( event.getType() == DownloadManagerStateEvent.ET_ATTRIBUTE_CHANGED ){
+		final int type = event.getType();
+		
+		if ( 	type == DownloadManagerStateEvent.ET_ATTRIBUTE_WRITTEN ||
+				type == DownloadManagerStateEvent.ET_ATTRIBUTE_WILL_BE_READ 	){
 			
 			String	name = (String)event.getData();
 			
-			if ( name.equals( DownloadManagerState.AT_CATEGORY )){
+			List	property_listeners_ref = property_listeners;
+			
+			final TorrentAttribute	attr = convertAttribute( name );
+			
+			if ( attr != null ){
 				
-				List	property_listeners_ref = property_listeners;
-				
-				final TorrentAttribute	attr = convertAttribute( name );
-				
-				if ( attr != null ){
+				for (int i=0;i<property_listeners_ref.size();i++){
 					
-					for (int i=0;i<property_listeners_ref.size();i++){
-						
-						try{						
-							((DownloadPropertyListener)property_listeners_ref.get(i)).propertyChanged(
-									this,
-									new DownloadPropertyEvent()
+					try{						
+						((DownloadPropertyListener)property_listeners_ref.get(i)).propertyChanged(
+								this,
+								new DownloadPropertyEvent()
+								{
+									public int
+									getType()
 									{
-										public int
-										getType()
-										{
-											return( DownloadPropertyEvent.PT_TORRENT_ATTRIBUTE );
-										}
-										
-										public Object
-										getData()
-										{
-											return( attr );
-										}
-									});
-	
-						}catch( Throwable e ){
-							
-							Debug.printStackTrace( e );
-						}
+										return( type==DownloadManagerStateEvent.ET_ATTRIBUTE_WRITTEN
+													?DownloadPropertyEvent.PT_TORRENT_ATTRIBUTE_WRITTEN
+													:DownloadPropertyEvent.PT_TORRENT_ATTRIBUTE_WILL_BE_READ	);
+									}
+									
+									public Object
+									getData()
+									{
+										return( attr );
+									}
+								});
+
+					}catch( Throwable e ){
+						
+						Debug.printStackTrace( e );
 					}
-				}			
-			}
+				}
+			}			
 		}
 	}
 	
@@ -779,7 +1026,7 @@ DownloadImpl
 	public void
 	torrentChanged()
 	{
-		TRTrackerClient	client = download_manager.getTrackerClient();
+		TRTrackerAnnouncer	client = download_manager.getTrackerClient();
 		
 		if ( client != null ){
 			
@@ -790,6 +1037,14 @@ DownloadImpl
 	public void
 	addTrackerListener(
 		DownloadTrackerListener	l )
+	{
+		addTrackerListener(l, true);
+	}
+
+	public void
+	addTrackerListener(
+		DownloadTrackerListener	l,
+		boolean immediateTrigger )
 	{
 		try{
 			tracker_listeners_mon.enter();
@@ -809,9 +1064,11 @@ DownloadImpl
 			tracker_listeners_mon.exit();
 		}
 		
-		l.announceResult( last_announce_result );
-		
-		l.scrapeResult( last_scrape_result );
+		if (immediateTrigger) {
+			l.announceResult( last_announce_result );
+			
+			l.scrapeResult( last_scrape_result );
+		}
 	}
 	
 	public void
@@ -922,6 +1179,90 @@ DownloadImpl
 		}
 	}
 	
+	public boolean
+	activateRequest(
+		final int		count )
+	{
+		DownloadActivationEvent event = 
+			new DownloadActivationEvent()
+		{
+			public Download 
+			getDownload() 
+			{
+				return( DownloadImpl.this );
+			}
+			
+			public int
+			getActivationCount()
+			{
+				return( count );
+			}
+		};
+		
+		for (Iterator it=activation_listeners.iterator();it.hasNext();){	
+				
+			try{
+				DownloadActivationListener	listener = (DownloadActivationListener)it.next();
+				
+				if ( listener.activationRequested( event )){
+					
+					return( true );
+				}
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
+		}
+		
+		return( false );
+	}
+	
+	public DownloadActivationEvent
+	getActivationState()
+	{
+		return( activation_state );
+	}
+	
+	public void
+	addActivationListener(
+		DownloadActivationListener		l )
+	{
+		try{
+			peer_listeners_mon.enter();
+		
+			activation_listeners.add( l );
+			
+			if ( activation_listeners.size() == 1 ){
+				
+				download_manager.addActivationListener( this );
+			}
+		}finally{
+			
+			peer_listeners_mon.exit();
+		}
+	}
+	
+	public void
+	removeActivationListener(
+		DownloadActivationListener		l )
+	{
+		try{
+			peer_listeners_mon.enter();
+
+			activation_listeners.remove( l );
+			
+			if ( activation_listeners.size() == 0 ){
+				
+				download_manager.removeActivationListener( this );
+			}
+		}finally{
+			
+			peer_listeners_mon.exit();
+		}
+	}
+	
+	
+	
 	public void
 	peerManagerAdded(
 		PEPeerManager	manager )
@@ -965,6 +1306,41 @@ DownloadImpl
  		return( PeerManagerImpl.getPeerManager( pm));
  	}
  	
+	public DiskManager
+	getDiskManager()
+	{
+		PeerManager	pm = getPeerManager();
+		
+		if ( pm != null ){
+			
+			return( pm.getDiskManager());
+		}
+		
+		return( null );
+	}
+	
+	
+	
+	public DiskManagerFileInfo[]
+	getDiskManagerFileInfo()
+	{
+		org.gudy.azureus2.core3.disk.DiskManagerFileInfo[] info = download_manager.getDiskManagerFileInfo();
+		
+		if ( info == null ){
+			
+			return( new DiskManagerFileInfo[0] );
+		}
+		
+		DiskManagerFileInfo[]	res = new DiskManagerFileInfo[info.length];
+		
+		for (int i=0;i<res.length;i++){
+			
+			res[i] = new DiskManagerFileInfoImpl( this, info[i] );
+		}
+		
+		return( res );
+	}
+	
 	public void
 	peerAdded(
 		PEPeer 	peer )
@@ -997,13 +1373,13 @@ DownloadImpl
 	setMaximumDownloadKBPerSecond(
 		int		kb )
  	{
- 		download_manager.getStats().setMaxDownloadKBSpeed( kb<0?0:kb );
+ 		download_manager.getStats().setDownloadRateLimitBytesPerSecond( kb < 0 ? 0 : kb*1024 );
  	}
   	
   	public int
 	getMaximumDownloadKBPerSecond()
   	{
-  		return( download_manager.getStats().getMaxDownloadKBSpeed());
+  		return( download_manager.getStats().getDownloadRateLimitBytesPerSecond() /1024 );
   	}
     
   	public int getUploadRateLimitBytesPerSecond() {
@@ -1013,25 +1389,96 @@ DownloadImpl
   	public void setUploadRateLimitBytesPerSecond( int max_rate_bps ) {
       download_manager.getStats().setUploadRateLimitBytesPerSecond( max_rate_bps );
   	}
+
+  public int getSeedingRank() {
+    return download_manager.getSeedingRank();
+  }
     
+	public void setSeedingRank(int rank) {
+		download_manager.setSeedingRank(rank);
+	}
   	
 	public String
 	getSavePath()
  	{
-		return( download_manager.getTorrentSaveDirAndFile());
+		return( download_manager.getSaveLocation().toString());
  	}
 	
+	public void
+  	moveDataFiles(
+  		File	new_parent_dir )
+  	
+  		throws DownloadException
+  	{
+ 		try{
+ 			download_manager.moveDataFiles( new_parent_dir );
+ 			
+ 		}catch( DownloadManagerException e ){
+ 			
+ 			throw( new DownloadException("move operation failed", e ));
+ 		}
+  	}
+	
+	public void renameDownload(String new_name) throws DownloadException {
+		try {download_manager.renameDownload(new_name);}
+		catch (DownloadManagerException e) {
+			throw new DownloadException("rename operation failed", e);
+		}
+	}
+  	
+  	public void
+  	moveTorrentFile(
+  		File	new_parent_dir )
+  	
+  		throws DownloadException
+ 	{
+		try{
+ 			download_manager.moveTorrentFile( new_parent_dir );
+ 			
+ 		}catch( DownloadManagerException e ){
+ 			
+ 			throw( new DownloadException("move operation failed", e ));
+ 		}  	
+ 	}
+  	
  	public void
 	requestTrackerAnnounce()
  	{
- 		download_manager.checkTracker();
+ 		download_manager.requestTrackerAnnounce( false );
  	}
  	
+	public void
+	requestTrackerAnnounce(
+		boolean		immediate )
+	{
+		download_manager.requestTrackerAnnounce( immediate );
+	}
+	
+	public void
+	requestTrackerScrape(
+		boolean		immediate )
+	{
+		download_manager.requestTrackerScrape( immediate );
+	}
+	
   public byte[] getDownloadPeerId() {
-    return download_manager.getTrackerClient().getPeerId();
+    TRTrackerAnnouncer announcer = download_manager.getTrackerClient();
+    if(announcer == null) return null;
+    return announcer.getPeerId();
   }
   
   
+  public boolean isMessagingEnabled() {  return download_manager.isAZMessagingEnabled();  }
+
+  public void setMessagingEnabled( boolean enabled ) {
+    download_manager.setAZMessagingEnabled( enabled );
+  }
+  
+  
+  public void setSessionAuthenticator( SessionAuthenticator auth ) {
+    //TODO
+  }
+  
   
  	// Deprecated methods
 
@@ -1045,4 +1492,20 @@ DownloadImpl
 
   public void setPriority(int priority) {
   }
+
+  // Pass LogRelation off to core objects
+
+	/* (non-Javadoc)
+	 * @see org.gudy.azureus2.core3.logging.LogRelation#getLogRelationText()
+	 */
+	public String getRelationText() {
+		return propogatedRelationText(download_manager);
+	}
+
+	/* (non-Javadoc)
+	 * @see org.gudy.azureus2.core3.logging.LogRelation#getQueryableInterfaces()
+	 */
+	public Object[] getQueryableInterfaces() {
+		return new Object[] { download_manager };
+	}
 }
