@@ -3,7 +3,7 @@
  * Created : 11 mar. 2004
  * By      : TuxPaper
  * 
- * Copyright (C) 2004 Aelitis SARL, All rights Reserved
+ * Copyright (C) 2004, 2005, 2006 Aelitis SAS, All rights Reserved
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * AELITIS, SARL au capital de 30,000 euros,
+ * AELITIS, SAS au capital de 46,603.30 euros,
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  */
 
@@ -44,16 +44,17 @@ import org.eclipse.swt.events.MouseEvent;
 
 import com.aelitis.azureus.core.*;
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
-import org.gudy.azureus2.plugins.ui.config.ConfigSectionSWT;
 import org.gudy.azureus2.ui.swt.config.*;
+import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.ipfilter.IpFilter;
+import org.gudy.azureus2.core3.ipfilter.IpFilterManager;
 import org.gudy.azureus2.core3.ipfilter.IpRange;
-import org.gudy.azureus2.core3.logging.LGLogger;
+import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.util.DisplayFormatters;
 
-public class ConfigSectionIPFilter implements ConfigSectionSWT {
+public class ConfigSectionIPFilter implements UISWTConfigSection {
   AzureusCore	azureus_core;
   
   IpFilter filter;
@@ -62,6 +63,8 @@ public class ConfigSectionIPFilter implements ConfigSectionSWT {
   
   FilterComparator comparator;
   
+  private boolean bIsCachingDescriptions = false;
+  
   class FilterComparator implements Comparator {
     
     boolean ascending = true;
@@ -120,40 +123,61 @@ public class ConfigSectionIPFilter implements ConfigSectionSWT {
       if (filter != null)
       	filter.save();
     }catch( Exception e ){
-    	LGLogger.logUnrepeatableAlert("Save of filter file fails", e);
+    	Logger.log(new LogAlert(LogAlert.UNREPEATABLE,
+					"Save of filter file fails", e));
     }
   }
 
   public void configSectionDelete() {
-    
+  	if (bIsCachingDescriptions) {
+	    IpFilterManager ipFilterManager = azureus_core.getIpFilterManager();
+	  	ipFilterManager.clearDescriptionCache();
+	  	bIsCachingDescriptions = false;
+  	}
   }
 
   public Composite configSectionCreate(final Composite parent) {
     GridData gridData;
 
-    filter = azureus_core.getIpFilterManager().getIPFilter();
+    int userMode = COConfigurationManager.getIntParameter("User Mode");
+
+    final IpFilterManager ipFilterManager = azureus_core.getIpFilterManager();
+    filter = ipFilterManager.getIPFilter();
+    
 
     Composite gFilter = new Composite(parent, SWT.NULL);
     GridLayout layout = new GridLayout();
-    layout.numColumns = 1;
+		layout.marginHeight = 0;
+		layout.marginWidth = 0;
     gFilter.setLayout(layout);
     gridData = new GridData(GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL);
     gFilter.setLayoutData(gridData);
     
+    
     percentage_blocked  = new Label(gFilter, SWT.NULL);
     gridData = new GridData(GridData.FILL_HORIZONTAL);
+	gridData.horizontalSpan = 3;
+
     percentage_blocked.setLayoutData(gridData);
 
     setPercentageBlocked();
     
     // start controls
-    BooleanParameter enabled = new BooleanParameter(gFilter, "Ip Filter Enabled",true); //$NON-NLS-1$
-    Messages.setLanguageText(enabled.getControl(), "ConfigView.section.ipfilter.enable"); //$NON-NLS-1$
 
+    	// row: enable filter + allow/deny
+    
+	gridData = new GridData();
 
-    BooleanParameter deny = new BooleanParameter(gFilter, "Ip Filter Allow",false); //$NON-NLS-1$
+    BooleanParameter enabled = new BooleanParameter(gFilter, "Ip Filter Enabled",true);
+	enabled.setLayoutData( gridData ); 
+    Messages.setLanguageText(enabled.getControl(), "ConfigView.section.ipfilter.enable");
+
+	gridData = new GridData();
+
+    BooleanParameter deny = new BooleanParameter(gFilter, "Ip Filter Allow",false);
+	deny.setLayoutData( gridData ); 
     Messages.setLanguageText(deny.getControl(), "ConfigView.section.ipfilter.allow");
-
+  
     deny.addChangeListener(
     	new ParameterChangeListener()
 		{
@@ -166,12 +190,60 @@ public class ConfigSectionIPFilter implements ConfigSectionSWT {
 			}
 		});
     
+    	// row persist banning
+    
+	gridData = new GridData();
+
+  BooleanParameter persist_bad_data_banning = new BooleanParameter(gFilter, "Ip Filter Banning Persistent",true);
+  persist_bad_data_banning.setLayoutData( gridData );
+  Messages.setLanguageText(persist_bad_data_banning.getControl(), "ConfigView.section.ipfilter.persistblocking");
+
+    	// row block bad + group ban
+    
+	gridData = new GridData();
+	
     BooleanParameter enable_bad_data_banning = new BooleanParameter(gFilter, "Ip Filter Enable Banning",true);
+	enable_bad_data_banning.setLayoutData( gridData );
     Messages.setLanguageText(enable_bad_data_banning.getControl(), "ConfigView.section.ipfilter.enablebanning");
 
+  	// description scratch file
+		if (userMode > 0) {
+			gridData = new GridData();
+			BooleanParameter enableDesc = new BooleanParameter(gFilter,
+					"Ip Filter Enable Description Cache", true);
+			enableDesc.setLayoutData(gridData);
+			Messages.setLanguageText(enableDesc.getControl(),
+					"ConfigView.section.ipfilter.enable.descriptionCache");
+		}
+      
+    	// block banning
+
+    Composite cLine = new Composite(gFilter, SWT.NULL);
+		layout = new GridLayout();
+		layout.marginHeight = 0;
+		layout.marginWidth = 0;
+		layout.numColumns = 2;
+		cLine.setLayout(layout);
+
+		Label block_label = new Label(cLine, SWT.NULL);
+		Messages.setLanguageText(block_label,
+				"ConfigView.section.ipfilter.blockbanning");
+
+		IntParameter block_banning = new IntParameter(cLine,
+				"Ip Filter Ban Block Limit");
+		gridData = new GridData();
+		gridData.widthHint = 30;
+		block_banning.setLayoutData(gridData);
+
+		enable_bad_data_banning
+				.setAdditionalActionPerformer(new ChangeSelectionActionPerformer(
+						new Control[] { block_banning.getControl(), block_label }));
+	
+		// table
+	
     table = new Table(gFilter, SWT.SINGLE | SWT.BORDER | SWT.FULL_SELECTION | SWT.VIRTUAL);
     String[] headers = { "ConfigView.section.ipfilter.description", "ConfigView.section.ipfilter.start", "ConfigView.section.ipfilter.end" };
-    int[] sizes = { 200, 110, 110 };
+    int[] sizes = { 110, 110, 110 };
     int[] aligns = { SWT.LEFT, SWT.CENTER, SWT.CENTER };
     for (int i = 0; i < headers.length; i++) {
       TableColumn tc = new TableColumn(table, aligns[i]);
@@ -192,6 +264,11 @@ public class ConfigSectionIPFilter implements ConfigSectionSWT {
         TableColumn tc = (TableColumn) e.widget;
         int field = ((Integer) tc.getData()).intValue();
         comparator.setField(field);
+        
+        if (field == FilterComparator.FIELD_NAME && !bIsCachingDescriptions) {
+        	ipFilterManager.cacheAllDescriptions();
+        	bIsCachingDescriptions = true;
+        }
         ipRanges = getSortedRanges(filter.getRanges());
         table.setItemCount(ipRanges.length);
         table.clearAll();
@@ -206,17 +283,20 @@ public class ConfigSectionIPFilter implements ConfigSectionSWT {
 
     table.setHeaderVisible(true);
 
-    gridData = new GridData(GridData.FILL_HORIZONTAL);
-    gridData.heightHint = 400;
+    gridData = new GridData(GridData.FILL_BOTH);
+    gridData.heightHint = table.getHeaderHeight() * 2;
+		gridData.widthHint = 200;
     table.setLayoutData(gridData);
 
+	gridData = new GridData();
+
     Composite cArea = new Composite(gFilter, SWT.NULL);
     layout = new GridLayout();
     layout.marginHeight = 0;
     layout.marginWidth = 0;
     layout.numColumns = 3;
     cArea.setLayout(layout);
-    cArea.setLayoutData(new GridData());
+    cArea.setLayoutData(gridData);
 
     Button add = new Button(cArea, SWT.PUSH);
     gridData = new GridData(GridData.CENTER);
@@ -283,6 +363,12 @@ public class ConfigSectionIPFilter implements ConfigSectionSWT {
       public void handleEvent(Event event) {
         TableItem item = (TableItem) event.item;
 		int index = table.indexOf (item);
+		
+			// seems we can get -1 here (see bug 1219314 )
+		
+		if ( index < 0 || index >= ipRanges.length ){
+			return;
+		}
 		IpRange range = ipRanges[index];		
         item.setText(0, range.getDescription());
         item.setText(1, range.getStartIp());
@@ -328,13 +414,13 @@ public class ConfigSectionIPFilter implements ConfigSectionSWT {
   }
 
   public void editRange(IpRange range) {
-    new IpFilterEditor(azureus_core,table.getDisplay(), table, range);
+    new IpFilterEditor(azureus_core,table.getShell(), table, range);
     noChange = false;
     //refresh();
   }
 
   public void addRange() {
-    new IpFilterEditor(azureus_core,table.getDisplay(), table, null);
+    new IpFilterEditor(azureus_core,table.getShell(), table, null);
     //noChange = false;
     //refresh();
   }
