@@ -1,7 +1,7 @@
 /*
  * Created on 20-May-2004
  * Created by Paul Gardner
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -29,12 +29,11 @@ package org.gudy.azureus2.update;
 
 import java.util.*;
 import java.net.*;
-import java.nio.ByteBuffer;
-import java.nio.channels.SocketChannel;
 import java.io.*;
 
 import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.core3.logging.*;
+import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.config.*;
 import org.gudy.azureus2.core3.html.*;
 
@@ -42,14 +41,17 @@ import org.gudy.azureus2.plugins.*;
 import org.gudy.azureus2.plugins.logging.*;
 import org.gudy.azureus2.plugins.update.*;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.*;
+import org.gudy.azureus2.pluginsimpl.local.update.UpdateManagerImpl;
+
+import com.aelitis.azureus.core.versioncheck.*;
 
-import com.aelitis.azureus.core.clientstats.*;
 
 public class 
 CoreUpdateChecker
 	implements Plugin, UpdatableComponent
 {
 	public static final String	LATEST_VERSION_PROPERTY	= "latest_version";
+	public static final String	MESSAGE_PROPERTY		= "message";
 	
 	public static final int	RD_GET_DETAILS_RETRIES	= 3;
 	public static final int	RD_GET_MIRRORS_RETRIES	= 3;
@@ -64,6 +66,8 @@ CoreUpdateChecker
 	protected LoggerChannel					log;
 	protected ResourceDownloaderListener	rd_logger;
 	
+	protected boolean						first_check		= true;
+	
 	public static void
 	doUsageStats()
 	{
@@ -80,17 +84,14 @@ CoreUpdateChecker
 	doUsageStatsSupport()
 	{
 		try{
-				// don't do any kind of reporting if the user doesn't want to
+			Map decoded = VersionCheckClient.getSingleton().getVersionCheckInfo(
+			  			first_check?VersionCheckClient.REASON_UPDATE_CHECK_START:VersionCheckClient.REASON_UPDATE_CHECK_PERIODIC);
+    	  
+			displayUserMessage( decoded );
 			
-			if ( COConfigurationManager.getBooleanParameter("Send Version Info")){
-				
-        performStatsServerUpdate();
-				
-				log.log( "Anonymous ID usage report ok" );
-			}
-		}catch( Throwable e ){
+		}finally{
 			
-			log.log( "Anonymous ID usage report fails", e );
+			  first_check	= false;
 		}
 	}
 
@@ -146,16 +147,12 @@ CoreUpdateChecker
 			String	current_version = plugin_interface.getAzureusVersion();
 			
 			log.log( "Update check starts: current = " + current_version );
-									
-			boolean	TESTING = false;	// !!!!TODO: REMOVE THIS
-			
-			if ( TESTING ){
-			
-				System.out.println( "CoreUpdater: !!!! Testing mode !!!!" );
-				
-			}
-					
-			Map	decoded = performStatsServerUpdate();
+													
+			Map	decoded = VersionCheckClient.getSingleton().getVersionCheckInfo(
+		  			first_check?VersionCheckClient.REASON_UPDATE_CHECK_START:VersionCheckClient.REASON_UPDATE_CHECK_PERIODIC);
+
+      
+			displayUserMessage( decoded );
 			
 			String latest_version 			= null;
 			String latest_file_name			= null;
@@ -186,40 +183,18 @@ CoreUpdateChecker
 			
 			log.log( msg );
 			
-			boolean	latest_is_cvs	= Constants.isCVSVersion( latest_version );
-			String	latest_base		= Constants.getBaseVersion( latest_version );
-			
-			String	current_base	= Constants.getBaseVersion();
-				
-				// currently we upgrade from, for example
-				//  1) 2.0.8.4     -> 2.0.8.6
-				//	2) 2.0.8.5_CVS -> 2.0.8.6
-				// but not to a CVS version (also currently never reported as latest...)
-				
-			if ( latest_is_cvs && !TESTING ){
+			if ( !shouldUpdate( current_version, latest_version )){
 				
 				return;
 			}
-
-			if ( Constants.compareVersions( current_base, latest_base ) >= 0 && !TESTING){
 				
-				return;
-			}
-			
 			final String	f_latest_version	= latest_version;
 			final String	f_latest_file_name	= latest_file_name;
 			
 
 			ResourceDownloader[]	primary_mirrors;
-			
-			if ( TESTING ){
 				
-				primary_mirrors = getTestMirrors();
-				
-			}else{
-				
-				primary_mirrors = getPrimaryDownloaders( latest_file_name );
-			}
+			primary_mirrors = getPrimaryDownloaders( latest_file_name );
 
 				// the download hierarchy is primary mirrors first (randomised alternate)
 				// then backup mirrors (randomised alternate)
@@ -256,19 +231,6 @@ CoreUpdateChecker
 			
 			top_downloader.getSize();		
 							
-			top_downloader.addListener( 
-					new ResourceDownloaderAdapter()
-					{
-						public boolean
-						completed(
-							final ResourceDownloader	downloader,
-							InputStream					data )
-						{	
-							installUpdate( checker, downloader, f_latest_version, data );
-									
-							return( true );
-						}
-					});
 
 			byte[]	info_b = (byte[])decoded.get( "info" );
 			
@@ -296,13 +258,27 @@ CoreUpdateChecker
 				desc = new String[]{"Core Azureus Version", info };
 			}
 			
-			checker.addUpdate(
+			final Update update = 
+				checker.addUpdate(
 						"Core Azureus Version",
 						desc,
 						latest_version,
 						top_downloader,
 						Update.RESTART_REQUIRED_YES );
 			
+			top_downloader.addListener( 
+					new ResourceDownloaderAdapter()
+					{
+						public boolean
+						completed(
+							final ResourceDownloader	downloader,
+							InputStream					data )
+						{	
+							installUpdate( checker, update, downloader, f_latest_version, data );
+									
+							return( true );
+						}
+					});
 		}catch( Throwable e ){
 			
 			log.log( e );
@@ -314,108 +290,12 @@ CoreUpdateChecker
 		}finally{
 			
 			checker.completed();
+			
+			first_check = false;
 		}
 	}
 	
-  
-  
-  
-  /**
-   * Connect to stats server and send info update.
-   * @return reply message from server
-   * @throws Exception
-   */
-  protected Map performStatsServerUpdate() throws Exception {
-    SocketChannel channel = null;
-    
-    try{
-      channel = SocketChannel.open();
-      channel.configureBlocking( true );
-      channel.connect( new InetSocketAddress( "azureus.aelitis.com", 6868 ) );
-      channel.finishConnect();
-    
-      ByteBuffer message = ByteBuffer.wrap( BEncoder.encode( constructStatsServerMessage() ) );
-    
-      StreamEncoder encoder = new StreamEncoder( "AZH", message );
-    
-      while( true ) {  //send message
-        if( encoder.encode( channel ) ) {
-          break;
-        }
-      }
-    
-      StreamDecoder decoder = new StreamDecoder( "AZR" );
-    
-      ByteBuffer reply;
-      while( true ) {  //receive reply
-        reply = decoder.decode( channel );
-        if( reply != null ) {
-          break;
-        }
-      }
-    
-      Map reply_message = BDecoder.decode( reply.array() );
-    
-      displayUserMessage( reply_message );
       
-      return reply_message;
-    }
-    finally {
-      if( channel != null )  channel.close();
-    }
-  }
-  
-  
-  /**
-   * Get the message to be sent to the stats server
-   * @return map to be bencoded
-   */
-  private Map constructStatsServerMessage() {
-    Map message = new HashMap();
-    
-    String id = COConfigurationManager.getStringParameter("ID",null);
-    
-    if ( id != null && COConfigurationManager.getBooleanParameter("Send Version Info")){
-      
-      message.put( "id", id );
-      message.put( "version", Constants.AZUREUS_VERSION );
-      message.put( "os", Constants.OSName );
-      
-      
-      String  java_version = System.getProperty("java.version");
-      if ( java_version == null ){  java_version = "unknown";  }
-      message.put( "java", java_version );
-      
-      
-      String  java_vendor = System.getProperty( "java.vm.vendor" );
-      if ( java_vendor == null ){   java_vendor = "unknown";  }
-      message.put( "javavendor", java_vendor );
-      
-      
-      long  max_mem = Runtime.getRuntime().maxMemory()/(1024*1024);
-      message.put( "javamx", new Long( max_mem ) );
-      
-      
-      //installed plugin IDs
-      PluginInterface[] plugins = plugin_interface.getPluginManager().getPluginInterfaces();
-      List pids = new ArrayList();
-      for (int i=0;i<plugins.length;i++){
-        String  pid = plugins[i].getPluginID();
-        
-          // filter out built-in and core ones
-        if (  !pid.startsWith( "<" ) && 
-            !pid.startsWith( "azupdater" ) &&
-            !pid.startsWith( "azplatform" ) &&
-            !pids.contains( pid )){
-        
-          pids.add( pid );
-        }
-      }
-      message.put( "plugins", pids );
-    }
-    
-    return message;
-  }
   
   
   /**
@@ -428,7 +308,7 @@ CoreUpdateChecker
     try{
       byte[]  message = (byte[])reply.get( "message" );
           
-      if ( message != null ){
+      if ( message != null && message.length > 0 ){
         
         String  s_message = new String(message);
         
@@ -436,17 +316,39 @@ CoreUpdateChecker
         
         if ( !s_message.equals( last )){
           
-          int   alert_type    = LGLogger.AT_WARNING;
+          byte[]	signature = (byte[])reply.get( "message_sig" );
+          
+          if ( signature == null ){
+        	  
+        	  Logger.log( new LogEvent( LogIDs.LOGGER, "Signature missing from message" ));
+        	  
+        	  return;
+          }
+          
+          try{
+	          AEVerifier.verifyData( s_message, signature );
+	          
+          }catch( Throwable e ){
+        	  
+           	  Logger.log( new LogEvent( LogIDs.LOGGER, "Message signature check failed", e  ));
+           	      
+           	  return;
+          }
+          
+          int   alert_type    = LogAlert.AT_WARNING;
           String  alert_text    = s_message;
           
           if ( alert_text.startsWith("i:" )){
           
-            alert_type = LGLogger.AT_COMMENT;
+            alert_type = LogAlert.AT_INFORMATION;
             
             alert_text = alert_text.substring(2);
           }
           
-          LGLogger.logUnrepeatableAlert( alert_type, alert_text );
+          plugin_interface.getPluginProperties().setProperty( MESSAGE_PROPERTY, alert_text );
+
+
+          Logger.log(new LogAlert(LogAlert.UNREPEATABLE, alert_type, alert_text));
           
           COConfigurationManager.setParameter( "CoreUpdateChecker.lastmessage", s_message );
           
@@ -457,111 +359,7 @@ CoreUpdateChecker
       
       Debug.printStackTrace( e );
     }
-  }
-    
-  
-  
-  
-  /*
-  protected Map
-  doHTTPVersionDownload()
-  
-    throws Exception
-  {
-    URL url = getVersionURL();
-    
-    ResourceDownloader  rd = rdf.create( url );
-    
-    rd = rdf.getRetryDownloader( rd, RD_GET_DETAILS_RETRIES );
-    
-    if ( rd_logger != null ){
-      
-      rd.addListener( rd_logger );
-    }
-    
-    BufferedInputStream data = new BufferedInputStream(rd.download());
-    
-    Map decoded = BDecoder.decode(data);
-    
-    data.close();
-
-    displayUserMessage( decoded );
-    
-    return( decoded );
-  }
-  
-  
-	protected URL
-	getVersionURL()
-		throws MalformedURLException, UnsupportedEncodingException
-	{
-		String url_str = Constants.AELITIS_WEB_SITE + "version.php";
-		
-		String id = COConfigurationManager.getStringParameter("ID",null);
-		
-		if ( id != null && COConfigurationManager.getBooleanParameter("Send Version Info")){
-			
-			String	java_version = System.getProperty("java.version");
-			
-			if ( java_version == null ){
-				
-				java_version = "unknown";
-			}
-			
-			String	java_vendor	= System.getProperty( "java.vm.vendor" );
-			
-			if ( java_vendor == null ){
-				
-				java_vendor = "unknown";
-			}
-			
-			long	max_mem = Runtime.getRuntime().maxMemory()/(1024*1024);
-			
-			PluginInterface[] plugins = plugin_interface.getPluginManager().getPluginInterfaces();
-			
-			
-			List	pids = new ArrayList();
-			
-			for (int i=0;i<plugins.length;i++){
-				
-				String	pid = plugins[i].getPluginID();
-				
-					// filter out built-in and core ones
-				
-				if ( 	!pid.startsWith( "<" ) &&
-						!pid.startsWith( "azupdater" ) &&
-						!pid.startsWith( "azplatform" ) &&
-						!pids.contains( pid )){
-				
-					pids.add( pid );
-				}
-			}
-			
-			String	plugins_str = "";
-
-			for (int i=0;i<pids.size();i++){
-				
-				String	pid = (String)pids.get(i);
-				
-				plugins_str += ( plugins_str.length()==0?"":";")+pid ;
-			}
-			
-			url_str += "?id=" + id + 
-							"&version=" + Constants.AZUREUS_VERSION + 
-							"&os=" + URLEncoder.encode( Constants.OSName, Constants.BYTE_ENCODING).replaceAll("\\+", "%20") +
-							"&java=" + URLEncoder.encode( java_version, Constants.BYTE_ENCODING).replaceAll("\\+", "%20") + 
-							"&javavendor=" + URLEncoder.encode( java_vendor, Constants.BYTE_ENCODING).replaceAll("\\+", "%20") +
-							"&javamx=" + max_mem +
-							"&plugins=" + URLEncoder.encode( plugins_str, Constants.BYTE_ENCODING).replaceAll("\\+", "%20");
-			}
-		
-		return( new URL( url_str ));
-	}
-  */
-  
-  
-  
-  
+  } 
 	
 	protected ResourceDownloader[]
 	getPrimaryDownloaders(
@@ -603,8 +401,14 @@ CoreUpdateChecker
 						position = -1;
 						
 					}else{
+						
 						String mirror = page.substring(position, end);
-	 
+						
+						if ( mirror.endsWith("\"")){
+							
+							mirror = mirror.substring(0,mirror.length()-1);
+						}
+						
 						try{
 							res.add( new URL( "http://prdownloads.sourceforge.net" + mirror ));
 							
@@ -678,6 +482,7 @@ CoreUpdateChecker
 					try{
 						
 						res.add( new URL( mirror + latest_file_name ));
+						// res.add( new URL( "http://torrents.aelitis.com:88/torrents/Azureus2.4.0.2_signed.jar.torrent" ));
 						
 					}catch(Throwable e){
 						
@@ -696,7 +501,7 @@ CoreUpdateChecker
 			
 			URL	url =(URL)res.get(i);
 			
-			log.log( "    Primary mirror:" +url.toString());
+			log.log( "    Backup mirror:" +url.toString());
 			
 			ResourceDownloader dl = rdf.create( url );
 
@@ -708,38 +513,21 @@ CoreUpdateChecker
 		}
 		
 		return( dls );
-	}    
- 
-	protected ResourceDownloader[]
-	getTestMirrors()
-	{
-		try{
-			ResourceDownloader[]	dls = new ResourceDownloader[1];
-			
-			ResourceDownloader rd = rdf.create( new URL("http://66.90.75.92/suprnova//torrents/1822/DivX511-exe.torrent" ));
-			
-			rd = rdf.getSuffixBasedDownloader( rd );
-	
-			dls[0] = rd;
-	
-			return( dls );
-			
-		}catch( Throwable e ){
-			
-			Debug.printStackTrace( e );
-			
-			return( new ResourceDownloader[0]);
-		}
-	}    	
+	}       	
 
 	protected void
 	installUpdate(
 		UpdateChecker		checker,
+		Update				update,
 		ResourceDownloader	rd,
 		String				version,
 		InputStream			data )
 	{
 		try{
+			data = update.verifyData( data, true );
+
+			rd.reportActivity( "Data verified successfully" );
+			
 			String	temp_jar_name 	= "Azureus2_" + version + ".jar";
 			String	target_jar_name	= "Azureus2.jar";
 			
@@ -751,7 +539,7 @@ CoreUpdateChecker
 				
 				installer.addMoveAction( 
 					temp_jar_name,
-					installer.getInstallDir() + "/Azureus.app/Contents/Resources/Java/" + target_jar_name );        
+					installer.getInstallDir() + "/" + SystemProperties.getApplicationName() + ".app/Contents/Resources/Java/" + target_jar_name );        
 			}else{
 				
 				installer.addMoveAction( 
@@ -763,4 +551,113 @@ CoreUpdateChecker
 			rd.reportActivity("Update install failed:" + e.getMessage());
 		}
 	}
+	
+	protected static boolean
+	shouldUpdate(
+		String	current_version,
+		String	latest_version )
+	{
+		String	current_base	= Constants.getBaseVersion( current_version );
+		int		current_inc		= Constants.getIncrementalBuild( current_version );
+
+		String	latest_base	= Constants.getBaseVersion( latest_version );
+		int		latest_inc	= Constants.getIncrementalBuild( latest_version );
+			
+			// currently we upgrade from, for example
+			//  1) 2.4.0.0     -> 2.4.0.2
+			//	2) 2.4.0.1_CVS -> 2.4.0.2
+			//	3) 2.4.0.1_B12 -> 2.4.0.2  and 2.4.0.1_B14
+		
+			// but NOT
+			//  1) 2.4.0.0 	   -> 2.4.0.1_CVS or 2.4.0.1_B23
+			//  2) 2.4.0.1_CVS -> 2.4.0.1_B23
+		
+			// for inc values: 0 = not CVS, -1 = _CVS, > 0 = Bnn
+
+		int	major_comp = Constants.compareVersions( current_base, latest_base );
+		
+		if ( major_comp < 0 && latest_inc == 0 ){
+			
+			return( true );		// latest is higher version and not CVS
+		}
+		
+			// same version, both are B versions and latest B is more recent
+		
+		return( major_comp == 0 && current_inc > 0 && latest_inc > 0 && latest_inc > current_inc );
+	}
+	
+	public static void
+	main(
+		String[]	args )
+	{
+		String[][]	tests = {
+				{ "2.4.0.0", 		"2.4.0.2", 		"true" },
+				{ "2.4.0.1_CVS", 	"2.4.0.2", 		"true" },
+				{ "2.4.0.1_B12",	"2.4.0.2", 		"true" },
+				{ "2.4.0.1_B12", 	"2.4.0.1_B34",	"true" },
+				{ "2.4.0.1_B12", 	"2.4.0.1_B6",	"false" },
+				{ "2.4.0.0", 		"2.4.0.1_CVS",	"false" },
+				{ "2.4.0.0", 		"2.4.0.1_B12",	"false" },
+				{ "2.4.0.0", 		"2.4.0.0"	,	"false" },
+				{ "2.4.0.1_CVS", 	"2.4.0.1_CVS",	"false" },
+				{ "2.4.0.1_B2", 	"2.4.0.1_B2",	"false" },
+				{ "2.4.0.1_CVS", 	"2.4.0.1_B2",	"false" },
+				{ "2.4.0.1_B2", 	"2.4.0.1_CVS",	"false" },
+
+		};
+
+		for (int i=0;i<tests.length;i++){
+			
+			System.out.println( shouldUpdate(tests[i][0],tests[i][1]) + " / " + tests[i][2] );
+		}
+		
+		/*
+		AEDiagnostics.startup();
+		
+		CoreUpdateChecker	checker = new CoreUpdateChecker();
+		
+		checker.log = new LoggerImpl(null).getTimeStampedChannel("");
+		checker.rdf	= new ResourceDownloaderFactoryImpl();
+		checker.rd_logger = 
+			new ResourceDownloaderAdapter()
+			{
+				public void
+				reportActivity(
+					ResourceDownloader	downloader,
+					String				activity )
+				{
+					System.out.println( activity );
+				}
+				
+				public void
+				reportPercentComplete(
+					ResourceDownloader	downloader,
+					int					percentage )
+				{
+					System.out.println( "    % = " + percentage );
+				}
+			};
+			
+		ResourceDownloader[]	primaries = checker.getPrimaryDownloaders( "Azureus-2.0.3.0.jar" );
+		
+		for (int i=0;i<primaries.length;i++){
+			
+			System.out.println( "primary: " + primaries[i].getName());
+		}
+		
+		try{
+			ResourceDownloader	rd = primaries[0];
+			
+			rd.addListener( checker.rd_logger );
+			
+			rd.download();
+			
+			System.out.println( "done" );
+			
+		}catch( Throwable e ){
+			
+			e.printStackTrace();
+		}
+		*/
+	}
 }
