@@ -253,7 +253,7 @@ implements IrcListener
 	
 	public void systemMessage(String message) {
 		//From services?
-		if ((message.indexOf("This nickname is owned by someone else") == 0) || (message.indexOf("If this is your nickname, type /msg NickServ IDENTIFY <password>") == 0))
+		if ((message.indexOf("This nickname is owned by someone else") == 0) || (message.indexOf("If this is your nickname, type /msg NickServ IDENTIFY <password>") == 0))
 		{
 			doLog(5, message);
 			} else {
@@ -379,12 +379,17 @@ implements IrcListener
 	
 	//This is evil in it's purest form.
 	class flash extends Thread {
+		String message = locale_utils.getLocalisedMessageText("IrcClient.topicforchannel") + ": " + client.getTopic();
+		flash(String message) {
+			if (message != "")
+				this.message = message;
+        }
 		public void run() {
 			for (int i=0;i<20;i++) {
 				try {
-					topicBar.setText(6, locale_utils.getLocalisedMessageText("IrcClient.topicforchannel") + ": " + client.getTopic());
+					topicBar.setText(6, message);
 					Thread.sleep(100);
-					topicBar.setText(7, locale_utils.getLocalisedMessageText("IrcClient.topicforchannel") + ": " + client.getTopic());
+					topicBar.setText(7, message);
 					Thread.sleep(100);
 				}
 				catch (Exception ex)
@@ -396,8 +401,8 @@ implements IrcListener
 		}
 	}
 	
-	public void topicFlash() {
-		Thread bleh = new flash();
+	public void topicFlash(String message) {
+		Thread bleh = new flash(message);
 		bleh.start();
 	}
 	
@@ -429,7 +434,7 @@ implements IrcListener
 		//This string is what kills GCJ 1.4.2 compatibility \o/
 		return "(?i)(?<!<)\\b" + nick + "\\w*";
 	}
-	
+
 	//Run the local INFO in a new thread so it doesn't lag the one doing the GUI updating.
 	class myINFO extends Thread {
 		public void run() {
@@ -440,6 +445,26 @@ implements IrcListener
 		}
 	}
 	
+	class staticIP extends Thread {
+		public void run() {
+			systemMessage("Now assigning a static IP... (Use /dhcp to revert to DHCP)");
+			client.assignStatic();
+		}
+	}
+	
+	/*class runRUN extends Thread {
+		public void run() {
+			client.runRUN();
+		}
+	}*/
+	
+	class dhcpIP extends Thread {
+		public void run() {
+			systemMessage("Now reverting back to DHCP...");
+			client.assignDHCP();
+		}
+	}
+	
 	void sendMessage(String text) {
 		if (text.equals(""))
 		return;
@@ -455,7 +480,24 @@ implements IrcListener
 				String action = text.substring(4).trim();
 				client.sendAction(action);
 				action(client.getUserName(), action);
-				} else if(text.toLowerCase().startsWith("/msg ")) {
+				} else if (text.equalsIgnoreCase("/dhcp"))
+				{
+					Thread bleh = new dhcpIP();
+					bleh.start();
+				}
+				else if (text.equalsIgnoreCase("/static"))
+				{
+					Thread bleh = new staticIP();
+					bleh.start();
+				}
+				else if (text.equalsIgnoreCase("/set"))
+					client.acceptSET();
+				/*else if (text.equalsIgnoreCase("/run"))
+				{
+					Thread bleh = new runRUN();
+					bleh.start();
+				}*/
+				else if(text.toLowerCase().startsWith("/msg ")) {
 				StringTokenizer st = new StringTokenizer(text," ");
 				st.nextToken();
 				try {
@@ -478,8 +520,8 @@ implements IrcListener
 			}
 			else if(text.toLowerCase().startsWith("/join "))
 			{
-				String channel = text.substring(6).trim();
-				client.changeChannel(channel,channel + " (T)");
+				String[] parts = text.substring(6).split(" ");
+				client.changeChannel(parts[0], parts[0] + " (T)");
 			}
 			else if(text.toLowerCase().equals("/info"))
 			{
@@ -544,7 +586,7 @@ implements IrcListener
 					}
 					} else {
 					//Abuse control.
-					client.sendMessage(sender, "You do not have permission to do that.");
+					client.sendMessage(sender, IrcClient.permError);
 				}
 			}
 		}
@@ -555,7 +597,7 @@ implements IrcListener
 				this.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.topicforchannel") + " " + client.getChannel() + ": " + client.getTopic());
 				}
 			else {
-				client.sendMessage(sender, "You do not have permission to do that.");
+				client.sendMessage(sender, IrcClient.permError);
 			}
 		}
 		else
